# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ..base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)


class JistLaminarVolumetricLayeringInputSpec(CommandLineInputSpec):
    inInner = File(
        desc="Inner Distance Image (GM/WM boundary)", exists=True, argstr="--inInner %s"
    )
    inOuter = File(
        desc="Outer Distance Image (CSF/GM boundary)",
        exists=True,
        argstr="--inOuter %s",
    )
    inNumber = traits.Int(desc="Number of layers", argstr="--inNumber %d")
    inMax = traits.Int(
        desc="Max iterations for narrow band evolution", argstr="--inMax %d"
    )
    inMin = traits.Float(
        desc="Min change ratio for narrow band evolution", argstr="--inMin %f"
    )
    inLayering = traits.Enum(
        "distance-preserving",
        "volume-preserving",
        desc="Layering method",
        argstr="--inLayering %s",
    )
    inLayering2 = traits.Enum(
        "outward", "inward", desc="Layering direction", argstr="--inLayering2 %s"
    )
    incurvature = traits.Int(
        desc="curvature approximation scale (voxels)", argstr="--incurvature %d"
    )
    inratio = traits.Float(
        desc="ratio smoothing kernel size (voxels)", argstr="--inratio %f"
    )
    inpresmooth = traits.Enum(
        "true", "false", desc="pre-smooth cortical surfaces", argstr="--inpresmooth %s"
    )
    inTopology = traits.Enum(
        "26/6",
        "6/26",
        "18/6",
        "6/18",
        "6/6",
        "wcs",
        "wco",
        "no",
        desc="Topology",
        argstr="--inTopology %s",
    )
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outContinuous = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Continuous depth measurement",
        argstr="--outContinuous %s",
    )
    outDiscrete = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Discrete sampled layers",
        argstr="--outDiscrete %s",
    )
    outLayer = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Layer boundary surfaces",
        argstr="--outLayer %s",
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class JistLaminarVolumetricLayeringOutputSpec(TraitedSpec):
    outContinuous = File(desc="Continuous depth measurement", exists=True)
    outDiscrete = File(desc="Discrete sampled layers", exists=True)
    outLayer = File(desc="Layer boundary surfaces", exists=True)


class JistLaminarVolumetricLayering(SEMLikeCommandLine):
    """Volumetric Layering.

    Builds a continuous layering of the cortex following distance-preserving or volume-preserving
    models of cortical folding.

    References
    ----------
    Waehnert MD, Dinse J, Weiss M, Streicher MN, Waehnert P, Geyer S, Turner R, Bazin PL,
    Anatomically motivated modeling of cortical laminae, Neuroimage, 2013.

    """

    input_spec = JistLaminarVolumetricLayeringInputSpec
    output_spec = JistLaminarVolumetricLayeringOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.laminar.JistLaminarVolumetricLayering "
    _outputs_filenames = {
        "outContinuous": "outContinuous.nii",
        "outLayer": "outLayer.nii",
        "outDiscrete": "outDiscrete.nii",
    }
    _redirect_x = True


class JistBrainMgdmSegmentationInputSpec(CommandLineInputSpec):
    inMP2RAGE = File(desc="MP2RAGE T1 Map Image", exists=True, argstr="--inMP2RAGE %s")
    inMP2RAGE2 = File(
        desc="MP2RAGE T1-weighted Image", exists=True, argstr="--inMP2RAGE2 %s"
    )
    inPV = File(desc="PV / Dura Image", exists=True, argstr="--inPV %s")
    inMPRAGE = File(
        desc="MPRAGE T1-weighted Image", exists=True, argstr="--inMPRAGE %s"
    )
    inFLAIR = File(desc="FLAIR Image", exists=True, argstr="--inFLAIR %s")
    inAtlas = File(desc="Atlas file", exists=True, argstr="--inAtlas %s")
    inData = traits.Float(desc="Data weight", argstr="--inData %f")
    inCurvature = traits.Float(desc="Curvature weight", argstr="--inCurvature %f")
    inPosterior = traits.Float(desc="Posterior scale (mm)", argstr="--inPosterior %f")
    inMax = traits.Int(desc="Max iterations", argstr="--inMax %d")
    inMin = traits.Float(desc="Min change", argstr="--inMin %f")
    inSteps = traits.Int(desc="Steps", argstr="--inSteps %d")
    inTopology = traits.Enum(
        "26/6",
        "6/26",
        "18/6",
        "6/18",
        "6/6",
        "wcs",
        "wco",
        "no",
        desc="Topology",
        argstr="--inTopology %s",
    )
    inCompute = traits.Enum(
        "true", "false", desc="Compute posteriors", argstr="--inCompute %s"
    )
    inAdjust = traits.Enum(
        "true", "false", desc="Adjust intensity priors", argstr="--inAdjust %s"
    )
    inOutput = traits.Enum(
        "segmentation", "memberships", desc="Output images", argstr="--inOutput %s"
    )
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outSegmented = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Segmented Brain Image",
        argstr="--outSegmented %s",
    )
    outLevelset = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Levelset Boundary Image",
        argstr="--outLevelset %s",
    )
    outPosterior2 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Posterior Maximum Memberships (4D)",
        argstr="--outPosterior2 %s",
    )
    outPosterior3 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Posterior Maximum Labels (4D)",
        argstr="--outPosterior3 %s",
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class JistBrainMgdmSegmentationOutputSpec(TraitedSpec):
    outSegmented = File(desc="Segmented Brain Image", exists=True)
    outLevelset = File(desc="Levelset Boundary Image", exists=True)
    outPosterior2 = File(desc="Posterior Maximum Memberships (4D)", exists=True)
    outPosterior3 = File(desc="Posterior Maximum Labels (4D)", exists=True)


class JistBrainMgdmSegmentation(SEMLikeCommandLine):
    """MGDM Whole Brain Segmentation.

    Estimate brain structures from an atlas for a MRI dataset (multiple input combinations
    are possible).

    """

    input_spec = JistBrainMgdmSegmentationInputSpec
    output_spec = JistBrainMgdmSegmentationOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.brain.JistBrainMgdmSegmentation "
    _outputs_filenames = {
        "outSegmented": "outSegmented.nii",
        "outPosterior2": "outPosterior2.nii",
        "outPosterior3": "outPosterior3.nii",
        "outLevelset": "outLevelset.nii",
    }
    _redirect_x = True


class JistLaminarProfileGeometryInputSpec(CommandLineInputSpec):
    inProfile = File(desc="Profile Surface Image", exists=True, argstr="--inProfile %s")
    incomputed = traits.Enum(
        "thickness",
        "curvedness",
        "shape_index",
        "mean_curvature",
        "gauss_curvature",
        "profile_length",
        "profile_curvature",
        "profile_torsion",
        desc="computed measure",
        argstr="--incomputed %s",
    )
    inregularization = traits.Enum(
        "none", "Gaussian", desc="regularization", argstr="--inregularization %s"
    )
    insmoothing = traits.Float(desc="smoothing parameter", argstr="--insmoothing %f")
    inoutside = traits.Float(desc="outside extension (mm)", argstr="--inoutside %f")
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outResult = traits.Either(
        traits.Bool, File(), hash_files=False, desc="Result", argstr="--outResult %s"
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class JistLaminarProfileGeometryOutputSpec(TraitedSpec):
    outResult = File(desc="Result", exists=True)


class JistLaminarProfileGeometry(SEMLikeCommandLine):
    """Compute various geometric quantities for a cortical layers."""

    input_spec = JistLaminarProfileGeometryInputSpec
    output_spec = JistLaminarProfileGeometryOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.laminar.JistLaminarProfileGeometry "
    _outputs_filenames = {"outResult": "outResult.nii"}
    _redirect_x = True


class JistLaminarProfileCalculatorInputSpec(CommandLineInputSpec):
    inIntensity = File(
        desc="Intensity Profile Image", exists=True, argstr="--inIntensity %s"
    )
    inMask = File(desc="Mask Image (opt, 3D or 4D)", exists=True, argstr="--inMask %s")
    incomputed = traits.Enum(
        "mean",
        "stdev",
        "skewness",
        "kurtosis",
        desc="computed statistic",
        argstr="--incomputed %s",
    )
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outResult = traits.Either(
        traits.Bool, File(), hash_files=False, desc="Result", argstr="--outResult %s"
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class JistLaminarProfileCalculatorOutputSpec(TraitedSpec):
    outResult = File(desc="Result", exists=True)


class JistLaminarProfileCalculator(SEMLikeCommandLine):
    """Compute various moments for intensities mapped along a cortical profile."""

    input_spec = JistLaminarProfileCalculatorInputSpec
    output_spec = JistLaminarProfileCalculatorOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.laminar.JistLaminarProfileCalculator "
    _outputs_filenames = {"outResult": "outResult.nii"}
    _redirect_x = True


class MedicAlgorithmN3InputSpec(CommandLineInputSpec):
    inInput = File(desc="Input Volume", exists=True, argstr="--inInput %s")
    inSignal = traits.Float(
        desc="Default = min + 1, Values at less than threshold are treated as part of the background",
        argstr="--inSignal %f",
    )
    inMaximum = traits.Int(desc="Maximum number of Iterations", argstr="--inMaximum %d")
    inEnd = traits.Float(
        desc="Usually 0.01-0.00001, The measure used to terminate the iterations is the coefficient of variation of change in field estimates between successive iterations.",
        argstr="--inEnd %f",
    )
    inField = traits.Float(
        desc="Characteristic distance over which the field varies. The distance between adjacent knots in bspline fitting with at least 4 knots going in every dimension. The default in the dialog is one third the distance (resolution * extents) of the smallest dimension.",
        argstr="--inField %f",
    )
    inSubsample = traits.Float(
        desc="Usually between 1-32, The factor by which the data is subsampled to a lower resolution in estimating the slowly varying non-uniformity field. Reduce sampling in the finest sampling direction by the shrink factor.",
        argstr="--inSubsample %f",
    )
    inKernel = traits.Float(
        desc="Usually between 0.05-0.50, Width of deconvolution kernel used to sharpen the histogram. Larger values give faster convergence while smaller values give greater accuracy.",
        argstr="--inKernel %f",
    )
    inWeiner = traits.Float(desc="Usually between 0.0-1.0", argstr="--inWeiner %f")
    inAutomatic = traits.Enum(
        "true",
        "false",
        desc="If true determines the threshold by histogram analysis. If true a VOI cannot be used and the input threshold is ignored.",
        argstr="--inAutomatic %s",
    )
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outInhomogeneity = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Inhomogeneity Corrected Volume",
        argstr="--outInhomogeneity %s",
    )
    outInhomogeneity2 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Inhomogeneity Field",
        argstr="--outInhomogeneity2 %s",
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class MedicAlgorithmN3OutputSpec(TraitedSpec):
    outInhomogeneity = File(desc="Inhomogeneity Corrected Volume", exists=True)
    outInhomogeneity2 = File(desc="Inhomogeneity Field", exists=True)


class MedicAlgorithmN3(SEMLikeCommandLine):
    """Non-parametric Intensity Non-uniformity Correction, N3, originally by J.G. Sled."""

    input_spec = MedicAlgorithmN3InputSpec
    output_spec = MedicAlgorithmN3OutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run edu.jhu.ece.iacl.plugins.classification.MedicAlgorithmN3 "
    _outputs_filenames = {
        "outInhomogeneity2": "outInhomogeneity2.nii",
        "outInhomogeneity": "outInhomogeneity.nii",
    }
    _redirect_x = True


class JistLaminarROIAveragingInputSpec(CommandLineInputSpec):
    inIntensity = File(
        desc="Intensity Profile Image", exists=True, argstr="--inIntensity %s"
    )
    inROI = File(desc="ROI Mask", exists=True, argstr="--inROI %s")
    inROI2 = traits.Str(desc="ROI Name", argstr="--inROI2 %s")
    inMask = File(desc="Mask Image (opt, 3D or 4D)", exists=True, argstr="--inMask %s")
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outROI3 = traits.Either(
        traits.Bool, File(), hash_files=False, desc="ROI Average", argstr="--outROI3 %s"
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class JistLaminarROIAveragingOutputSpec(TraitedSpec):
    outROI3 = File(desc="ROI Average", exists=True)


class JistLaminarROIAveraging(SEMLikeCommandLine):
    """Compute an average profile over a given ROI."""

    input_spec = JistLaminarROIAveragingInputSpec
    output_spec = JistLaminarROIAveragingOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.laminar.JistLaminarROIAveraging "
    _outputs_filenames = {"outROI3": "outROI3"}
    _redirect_x = True


class MedicAlgorithmLesionToadsInputSpec(CommandLineInputSpec):
    inT1_MPRAGE = File(desc="T1_MPRAGE Image", exists=True, argstr="--inT1_MPRAGE %s")
    inT1_SPGR = File(desc="T1_SPGR Image", exists=True, argstr="--inT1_SPGR %s")
    inFLAIR = File(desc="FLAIR Image", exists=True, argstr="--inFLAIR %s")
    inAtlas = traits.Enum(
        "With Lesion", "No Lesion", desc="Atlas to Use", argstr="--inAtlas %s"
    )
    inOutput = traits.Enum(
        "hard segmentation",
        "hard segmentation+memberships",
        "cruise inputs",
        "dura removal inputs",
        desc="Output images",
        argstr="--inOutput %s",
    )
    inOutput2 = traits.Enum(
        "true",
        "false",
        desc="Output the hard classification using maximum membership (not neceesarily topologically correct)",
        argstr="--inOutput2 %s",
    )
    inCorrect = traits.Enum(
        "true", "false", desc="Correct MR field inhomogeneity.", argstr="--inCorrect %s"
    )
    inOutput3 = traits.Enum(
        "true",
        "false",
        desc="Output the estimated inhomogeneity field",
        argstr="--inOutput3 %s",
    )
    inAtlas2 = File(
        desc="Atlas File - With Lesions", exists=True, argstr="--inAtlas2 %s"
    )
    inAtlas3 = File(
        desc="Atlas File - No Lesion - T1 and FLAIR",
        exists=True,
        argstr="--inAtlas3 %s",
    )
    inAtlas4 = File(
        desc="Atlas File - No Lesion - T1 Only", exists=True, argstr="--inAtlas4 %s"
    )
    inMaximum = traits.Int(
        desc="Maximum distance from the interventricular WM boundary to downweight the lesion membership to avoid false postives",
        argstr="--inMaximum %d",
    )
    inMaximum2 = traits.Int(desc="Maximum Ventircle Distance", argstr="--inMaximum2 %d")
    inMaximum3 = traits.Int(
        desc="Maximum InterVentricular Distance", argstr="--inMaximum3 %d"
    )
    inInclude = traits.Enum(
        "true",
        "false",
        desc="Include lesion in WM class in hard classification",
        argstr="--inInclude %s",
    )
    inAtlas5 = traits.Float(
        desc="Controls the effect of the statistical atlas on the segmentation",
        argstr="--inAtlas5 %f",
    )
    inSmooting = traits.Float(
        desc="Controls the effect of neighberhood voxels on the membership",
        argstr="--inSmooting %f",
    )
    inMaximum4 = traits.Float(
        desc="Maximum amount of relative change in the energy function considered as the convergence criteria",
        argstr="--inMaximum4 %f",
    )
    inMaximum5 = traits.Int(desc="Maximum iterations", argstr="--inMaximum5 %d")
    inAtlas6 = traits.Enum(
        "rigid", "multi_fully_affine", desc="Atlas alignment", argstr="--inAtlas6 %s"
    )
    inConnectivity = traits.Enum(
        "(26,6)",
        "(6,26)",
        "(6,18)",
        "(18,6)",
        desc="Connectivity (foreground,background)",
        argstr="--inConnectivity %s",
    )
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outHard = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Hard segmentation",
        argstr="--outHard %s",
    )
    outHard2 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Hard segmentationfrom memberships",
        argstr="--outHard2 %s",
    )
    outInhomogeneity = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Inhomogeneity Field",
        argstr="--outInhomogeneity %s",
    )
    outMembership = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Membership Functions",
        argstr="--outMembership %s",
    )
    outLesion = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Lesion Segmentation",
        argstr="--outLesion %s",
    )
    outSulcal = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Sulcal CSF Membership",
        argstr="--outSulcal %s",
    )
    outCortical = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Cortical GM Membership",
        argstr="--outCortical %s",
    )
    outFilled = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Filled WM Membership",
        argstr="--outFilled %s",
    )
    outWM = traits.Either(
        traits.Bool, File(), hash_files=False, desc="WM Mask", argstr="--outWM %s"
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class MedicAlgorithmLesionToadsOutputSpec(TraitedSpec):
    outHard = File(desc="Hard segmentation", exists=True)
    outHard2 = File(desc="Hard segmentationfrom memberships", exists=True)
    outInhomogeneity = File(desc="Inhomogeneity Field", exists=True)
    outMembership = File(desc="Membership Functions", exists=True)
    outLesion = File(desc="Lesion Segmentation", exists=True)
    outSulcal = File(desc="Sulcal CSF Membership", exists=True)
    outCortical = File(desc="Cortical GM Membership", exists=True)
    outFilled = File(desc="Filled WM Membership", exists=True)
    outWM = File(desc="WM Mask", exists=True)


class MedicAlgorithmLesionToads(SEMLikeCommandLine):
    """Algorithm for simulataneous brain structures and MS lesion segmentation of MS Brains.

    The brain segmentation is topologically consistent and the algorithm can use multiple
    MR sequences as input data.

    References
    ----------
    N. Shiee, P.-L. Bazin, A.Z. Ozturk, P.A. Calabresi, D.S. Reich, D.L. Pham,
    "A Topology-Preserving Approach to the Segmentation of Brain Images with Multiple Sclerosis",
    NeuroImage, vol. 49, no. 2, pp. 1524-1535, 2010.

    """

    input_spec = MedicAlgorithmLesionToadsInputSpec
    output_spec = MedicAlgorithmLesionToadsOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run edu.jhu.ece.iacl.plugins.classification.MedicAlgorithmLesionToads "
    _outputs_filenames = {
        "outWM": "outWM.nii",
        "outHard": "outHard.nii",
        "outFilled": "outFilled.nii",
        "outMembership": "outMembership.nii",
        "outInhomogeneity": "outInhomogeneity.nii",
        "outCortical": "outCortical.nii",
        "outHard2": "outHard2.nii",
        "outLesion": "outLesion.nii",
        "outSulcal": "outSulcal.nii",
    }
    _redirect_x = True


class JistBrainMp2rageSkullStrippingInputSpec(CommandLineInputSpec):
    inSecond = File(
        desc="Second inversion (Inv2) Image", exists=True, argstr="--inSecond %s"
    )
    inT1 = File(desc="T1 Map (T1_Images) Image (opt)", exists=True, argstr="--inT1 %s")
    inT1weighted = File(
        desc="T1-weighted (UNI) Image (opt)", exists=True, argstr="--inT1weighted %s"
    )
    inFilter = File(desc="Filter Image (opt)", exists=True, argstr="--inFilter %s")
    inSkip = traits.Enum("true", "false", desc="Skip zero values", argstr="--inSkip %s")
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outBrain = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Brain Mask Image",
        argstr="--outBrain %s",
    )
    outMasked = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Masked T1 Map Image",
        argstr="--outMasked %s",
    )
    outMasked2 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Masked T1-weighted Image",
        argstr="--outMasked2 %s",
    )
    outMasked3 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Masked Filter Image",
        argstr="--outMasked3 %s",
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class JistBrainMp2rageSkullStrippingOutputSpec(TraitedSpec):
    outBrain = File(desc="Brain Mask Image", exists=True)
    outMasked = File(desc="Masked T1 Map Image", exists=True)
    outMasked2 = File(desc="Masked T1-weighted Image", exists=True)
    outMasked3 = File(desc="Masked Filter Image", exists=True)


class JistBrainMp2rageSkullStripping(SEMLikeCommandLine):
    """Estimate a brain mask for a MP2RAGE dataset.

    At least a T1-weighted or a T1 map image is required.

    """

    input_spec = JistBrainMp2rageSkullStrippingInputSpec
    output_spec = JistBrainMp2rageSkullStrippingOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.brain.JistBrainMp2rageSkullStripping "
    _outputs_filenames = {
        "outBrain": "outBrain.nii",
        "outMasked3": "outMasked3.nii",
        "outMasked2": "outMasked2.nii",
        "outMasked": "outMasked.nii",
    }
    _redirect_x = True


class JistCortexSurfaceMeshInflationInputSpec(CommandLineInputSpec):
    inLevelset = File(desc="Levelset Image", exists=True, argstr="--inLevelset %s")
    inSOR = traits.Float(desc="SOR Parameter", argstr="--inSOR %f")
    inMean = traits.Float(desc="Mean Curvature Threshold", argstr="--inMean %f")
    inStep = traits.Int(desc="Step Size", argstr="--inStep %d")
    inMax = traits.Int(desc="Max Iterations", argstr="--inMax %d")
    inLorentzian = traits.Enum(
        "true", "false", desc="Lorentzian Norm", argstr="--inLorentzian %s"
    )
    inTopology = traits.Enum(
        "26/6",
        "6/26",
        "18/6",
        "6/18",
        "6/6",
        "wcs",
        "wco",
        "no",
        desc="Topology",
        argstr="--inTopology %s",
    )
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outOriginal = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Original Surface",
        argstr="--outOriginal %s",
    )
    outInflated = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Inflated Surface",
        argstr="--outInflated %s",
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class JistCortexSurfaceMeshInflationOutputSpec(TraitedSpec):
    outOriginal = File(desc="Original Surface", exists=True)
    outInflated = File(desc="Inflated Surface", exists=True)


class JistCortexSurfaceMeshInflation(SEMLikeCommandLine):
    """Inflates a cortical surface mesh.

    References
    ----------
    D. Tosun, M. E. Rettmann, X. Han, X. Tao, C. Xu, S. M. Resnick, D. Pham, and J. L. Prince,
    Cortical Surface Segmentation and Mapping, NeuroImage, vol. 23, pp. S108--S118, 2004.

    """

    input_spec = JistCortexSurfaceMeshInflationInputSpec
    output_spec = JistCortexSurfaceMeshInflationOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.cortex.JistCortexSurfaceMeshInflation "
    _outputs_filenames = {"outOriginal": "outOriginal", "outInflated": "outInflated"}
    _redirect_x = True


class RandomVolInputSpec(CommandLineInputSpec):
    inSize = traits.Int(desc="Size of Volume in X direction", argstr="--inSize %d")
    inSize2 = traits.Int(desc="Size of Volume in Y direction", argstr="--inSize2 %d")
    inSize3 = traits.Int(desc="Size of Volume in Z direction", argstr="--inSize3 %d")
    inSize4 = traits.Int(desc="Size of Volume in t direction", argstr="--inSize4 %d")
    inStandard = traits.Int(
        desc="Standard Deviation for Normal Distribution", argstr="--inStandard %d"
    )
    inLambda = traits.Float(
        desc="Lambda Value for Exponential Distribution", argstr="--inLambda %f"
    )
    inMaximum = traits.Int(desc="Maximum Value", argstr="--inMaximum %d")
    inMinimum = traits.Int(desc="Minimum Value", argstr="--inMinimum %d")
    inField = traits.Enum(
        "Uniform", "Normal", "Exponential", desc="Field", argstr="--inField %s"
    )
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outRand1 = traits.Either(
        traits.Bool, File(), hash_files=False, desc="Rand1", argstr="--outRand1 %s"
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class RandomVolOutputSpec(TraitedSpec):
    outRand1 = File(desc="Rand1", exists=True)


class RandomVol(SEMLikeCommandLine):
    """Generate a volume of random scalars."""

    input_spec = RandomVolInputSpec
    output_spec = RandomVolOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run edu.jhu.bme.smile.demo.RandomVol "
    _outputs_filenames = {"outRand1": "outRand1.nii"}
    _redirect_x = True


class MedicAlgorithmImageCalculatorInputSpec(CommandLineInputSpec):
    inVolume = File(desc="Volume 1", exists=True, argstr="--inVolume %s")
    inVolume2 = File(desc="Volume 2", exists=True, argstr="--inVolume2 %s")
    inOperation = traits.Enum(
        "Add",
        "Subtract",
        "Multiply",
        "Divide",
        "Min",
        "Max",
        desc="Operation",
        argstr="--inOperation %s",
    )
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outResult = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Result Volume",
        argstr="--outResult %s",
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class MedicAlgorithmImageCalculatorOutputSpec(TraitedSpec):
    outResult = File(desc="Result Volume", exists=True)


class MedicAlgorithmImageCalculator(SEMLikeCommandLine):
    """Perform simple image calculator operations on two images.

    The operations include 'Add', 'Subtract', 'Multiply', and 'Divide'

    """

    input_spec = MedicAlgorithmImageCalculatorInputSpec
    output_spec = MedicAlgorithmImageCalculatorOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run edu.jhu.ece.iacl.plugins.utilities.math.MedicAlgorithmImageCalculator "
    _outputs_filenames = {"outResult": "outResult.nii"}
    _redirect_x = True


class JistBrainMp2rageDuraEstimationInputSpec(CommandLineInputSpec):
    inSecond = File(
        desc="Second inversion (Inv2) Image", exists=True, argstr="--inSecond %s"
    )
    inSkull = File(desc="Skull Stripping Mask", exists=True, argstr="--inSkull %s")
    inDistance = traits.Float(
        desc="Distance to background (mm)", argstr="--inDistance %f"
    )
    inoutput = traits.Enum(
        "dura_region",
        "boundary",
        "dura_prior",
        "bg_prior",
        "intens_prior",
        desc="Outputs an estimate of the dura / CSF boundary or an estimate of the entire dura region.",
        argstr="--inoutput %s",
    )
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outDura = traits.Either(
        traits.Bool, File(), hash_files=False, desc="Dura Image", argstr="--outDura %s"
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class JistBrainMp2rageDuraEstimationOutputSpec(TraitedSpec):
    outDura = File(desc="Dura Image", exists=True)


class JistBrainMp2rageDuraEstimation(SEMLikeCommandLine):
    """Filters a MP2RAGE brain image to obtain a probability map of dura matter."""

    input_spec = JistBrainMp2rageDuraEstimationInputSpec
    output_spec = JistBrainMp2rageDuraEstimationOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.brain.JistBrainMp2rageDuraEstimation "
    _outputs_filenames = {"outDura": "outDura.nii"}
    _redirect_x = True


class JistLaminarProfileSamplingInputSpec(CommandLineInputSpec):
    inProfile = File(desc="Profile Surface Image", exists=True, argstr="--inProfile %s")
    inIntensity = File(desc="Intensity Image", exists=True, argstr="--inIntensity %s")
    inCortex = File(desc="Cortex Mask (opt)", exists=True, argstr="--inCortex %s")
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outProfilemapped = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Profile-mapped Intensity Image",
        argstr="--outProfilemapped %s",
    )
    outProfile2 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Profile 4D Mask",
        argstr="--outProfile2 %s",
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class JistLaminarProfileSamplingOutputSpec(TraitedSpec):
    outProfilemapped = File(desc="Profile-mapped Intensity Image", exists=True)
    outProfile2 = File(desc="Profile 4D Mask", exists=True)


class JistLaminarProfileSampling(SEMLikeCommandLine):
    """Sample some intensity image along a cortical profile across layer surfaces."""

    input_spec = JistLaminarProfileSamplingInputSpec
    output_spec = JistLaminarProfileSamplingOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.laminar.JistLaminarProfileSampling "
    _outputs_filenames = {
        "outProfile2": "outProfile2.nii",
        "outProfilemapped": "outProfilemapped.nii",
    }
    _redirect_x = True


class MedicAlgorithmMipavReorientInputSpec(CommandLineInputSpec):
    inSource = InputMultiPath(File, desc="Source", sep=";", argstr="--inSource %s")
    inTemplate = File(desc="Template", exists=True, argstr="--inTemplate %s")
    inNew = traits.Enum(
        "Dicom axial",
        "Dicom coronal",
        "Dicom sagittal",
        "User defined",
        desc="New image orientation",
        argstr="--inNew %s",
    )
    inUser = traits.Enum(
        "Unknown",
        "Patient Right to Left",
        "Patient Left to Right",
        "Patient Posterior to Anterior",
        "Patient Anterior to Posterior",
        "Patient Inferior to Superior",
        "Patient Superior to Inferior",
        desc="User defined X-axis orientation (image left to right)",
        argstr="--inUser %s",
    )
    inUser2 = traits.Enum(
        "Unknown",
        "Patient Right to Left",
        "Patient Left to Right",
        "Patient Posterior to Anterior",
        "Patient Anterior to Posterior",
        "Patient Inferior to Superior",
        "Patient Superior to Inferior",
        desc="User defined Y-axis orientation (image top to bottom)",
        argstr="--inUser2 %s",
    )
    inUser3 = traits.Enum(
        "Unknown",
        "Patient Right to Left",
        "Patient Left to Right",
        "Patient Posterior to Anterior",
        "Patient Anterior to Posterior",
        "Patient Inferior to Superior",
        "Patient Superior to Inferior",
        desc="User defined Z-axis orientation (into the screen)",
        argstr="--inUser3 %s",
    )
    inUser4 = traits.Enum(
        "Axial",
        "Coronal",
        "Sagittal",
        "Unknown",
        desc="User defined Image Orientation",
        argstr="--inUser4 %s",
    )
    inInterpolation = traits.Enum(
        "Nearest Neighbor",
        "Trilinear",
        "Bspline 3rd order",
        "Bspline 4th order",
        "Cubic Lagrangian",
        "Quintic Lagrangian",
        "Heptic Lagrangian",
        "Windowed Sinc",
        desc="Interpolation",
        argstr="--inInterpolation %s",
    )
    inResolution = traits.Enum(
        "Unchanged",
        "Finest cubic",
        "Coarsest cubic",
        "Same as template",
        desc="Resolution",
        argstr="--inResolution %s",
    )
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outReoriented = InputMultiPath(
        File, desc="Reoriented Volume", sep=";", argstr="--outReoriented %s"
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class MedicAlgorithmMipavReorientOutputSpec(TraitedSpec):
    pass


class MedicAlgorithmMipavReorient(SEMLikeCommandLine):
    """Reorient a volume to a particular anatomical orientation."""

    input_spec = MedicAlgorithmMipavReorientInputSpec
    output_spec = MedicAlgorithmMipavReorientOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run edu.jhu.ece.iacl.plugins.utilities.volume.MedicAlgorithmMipavReorient "
    _outputs_filenames = {}
    _redirect_x = True


class MedicAlgorithmSPECTRE2010InputSpec(CommandLineInputSpec):
    inInput = File(
        desc="Input volume to be skullstripped.", exists=True, argstr="--inInput %s"
    )
    inAtlas = File(
        desc="SPECTRE atlas description file. A text file enumerating atlas files and landmarks.",
        exists=True,
        argstr="--inAtlas %s",
    )
    inInitial = traits.Int(
        desc="Erosion of the inital mask, which is based on the probability mask and the classification., The initial mask is ouput as the d0 volume at the conclusion of SPECTRE.",
        argstr="--inInitial %d",
    )
    inImage = traits.Enum(
        "T1_SPGR",
        "T1_ALT",
        "T1_MPRAGE",
        "T2",
        "FLAIR",
        desc="Set the image modality. MP-RAGE is recommended for most T1 sequence images.",
        argstr="--inImage %s",
    )
    inOutput = traits.Enum(
        "true",
        "false",
        desc="Determines if the output results are transformed back into the space of the original input image.",
        argstr="--inOutput %s",
    )
    inFind = traits.Enum(
        "true", "false", desc="Find Midsaggital Plane", argstr="--inFind %s"
    )
    inRun = traits.Enum(
        "true", "false", desc="Run Smooth Brain Mask", argstr="--inRun %s"
    )
    inResample = traits.Enum(
        "true",
        "false",
        desc="Determines if the data is resampled to be isotropic during the processing.",
        argstr="--inResample %s",
    )
    inInitial2 = traits.Float(
        desc="Initial probability threshold", argstr="--inInitial2 %f"
    )
    inMinimum = traits.Float(
        desc="Minimum probability threshold", argstr="--inMinimum %f"
    )
    inMMC = traits.Int(
        desc="The size of the dilation step within the Modified Morphological Closing.",
        argstr="--inMMC %d",
    )
    inMMC2 = traits.Int(
        desc="The size of the erosion step within the Modified Morphological Closing.",
        argstr="--inMMC2 %d",
    )
    inInhomogeneity = traits.Enum(
        "true",
        "false",
        desc="Set to false by default, this parameter will make FANTASM try to do inhomogeneity correction during it's iterative cycle.",
        argstr="--inInhomogeneity %s",
    )
    inSmoothing = traits.Float(argstr="--inSmoothing %f")
    inBackground = traits.Float(argstr="--inBackground %f")
    inOutput2 = traits.Enum(
        "true", "false", desc="Output Plane?", argstr="--inOutput2 %s"
    )
    inOutput3 = traits.Enum(
        "true", "false", desc="Output Split-Halves?", argstr="--inOutput3 %s"
    )
    inOutput4 = traits.Enum(
        "true", "false", desc="Output Segmentation on Plane?", argstr="--inOutput4 %s"
    )
    inDegrees = traits.Enum(
        "Rigid - 6",
        "Global rescale - 7",
        "Specific rescale - 9",
        "Affine - 12",
        desc="Degrees of freedom",
        argstr="--inDegrees %s",
    )
    inCost = traits.Enum(
        "Correlation ratio",
        "Least squares",
        "Normalized cross correlation",
        "Normalized mutual information",
        desc="Cost function",
        argstr="--inCost %s",
    )
    inRegistration = traits.Enum(
        "Trilinear",
        "Bspline 3rd order",
        "Bspline 4th order",
        "Cubic Lagrangian",
        "Quintic Lagrangian",
        "Heptic Lagrangian",
        "Windowed sinc",
        desc="Registration interpolation",
        argstr="--inRegistration %s",
    )
    inOutput5 = traits.Enum(
        "Trilinear",
        "Bspline 3rd order",
        "Bspline 4th order",
        "Cubic Lagrangian",
        "Quintic Lagrangian",
        "Heptic Lagrangian",
        "Windowed sinc",
        "Nearest Neighbor",
        desc="Output interpolation",
        argstr="--inOutput5 %s",
    )
    inApply = traits.Enum(
        "All", "X", "Y", "Z", desc="Apply rotation", argstr="--inApply %s"
    )
    inMinimum2 = traits.Float(desc="Minimum angle", argstr="--inMinimum2 %f")
    inMaximum = traits.Float(desc="Maximum angle", argstr="--inMaximum %f")
    inCoarse = traits.Float(desc="Coarse angle increment", argstr="--inCoarse %f")
    inFine = traits.Float(desc="Fine angle increment", argstr="--inFine %f")
    inMultiple = traits.Int(
        desc="Multiple of tolerance to bracket the minimum", argstr="--inMultiple %d"
    )
    inNumber = traits.Int(desc="Number of iterations", argstr="--inNumber %d")
    inNumber2 = traits.Int(
        desc="Number of minima from Level 8 to test at Level 4", argstr="--inNumber2 %d"
    )
    inUse = traits.Enum(
        "true",
        "false",
        desc="Use the max of the min resolutions of the two datasets when resampling",
        argstr="--inUse %s",
    )
    inSubsample = traits.Enum(
        "true", "false", desc="Subsample image for speed", argstr="--inSubsample %s"
    )
    inSkip = traits.Enum(
        "true",
        "false",
        desc="Skip multilevel search (Assume images are close to alignment)",
        argstr="--inSkip %s",
    )
    inMultithreading = traits.Enum(
        "true",
        "false",
        desc="Set to false by default, this parameter controls the multithreaded behavior of the linear registration.",
        argstr="--inMultithreading %s",
    )
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outOriginal = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="If Output in Original Space Flag is true then outputs the original input volume. Otherwise outputs the axialy reoriented input volume.",
        argstr="--outOriginal %s",
    )
    outStripped = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Skullstripped result of the input volume with just the brain.",
        argstr="--outStripped %s",
    )
    outMask = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Binary Mask of the skullstripped result with just the brain",
        argstr="--outMask %s",
    )
    outPrior = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Probability prior from the atlas registrations",
        argstr="--outPrior %s",
    )
    outFANTASM = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Tissue classification of of the whole input volume.",
        argstr="--outFANTASM %s",
    )
    outd0 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Initial Brainmask",
        argstr="--outd0 %s",
    )
    outMidsagittal = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Plane dividing the brain hemispheres",
        argstr="--outMidsagittal %s",
    )
    outSplitHalves = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Skullstripped mask of the brain with the hemispheres divided.",
        argstr="--outSplitHalves %s",
    )
    outSegmentation = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="2D image showing the tissue classification on the midsagittal plane",
        argstr="--outSegmentation %s",
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class MedicAlgorithmSPECTRE2010OutputSpec(TraitedSpec):
    outOriginal = File(
        desc="If Output in Original Space Flag is true then outputs the original input volume. Otherwise outputs the axialy reoriented input volume.",
        exists=True,
    )
    outStripped = File(
        desc="Skullstripped result of the input volume with just the brain.",
        exists=True,
    )
    outMask = File(
        desc="Binary Mask of the skullstripped result with just the brain", exists=True
    )
    outPrior = File(desc="Probability prior from the atlas registrations", exists=True)
    outFANTASM = File(
        desc="Tissue classification of of the whole input volume.", exists=True
    )
    outd0 = File(desc="Initial Brainmask", exists=True)
    outMidsagittal = File(desc="Plane dividing the brain hemispheres", exists=True)
    outSplitHalves = File(
        desc="Skullstripped mask of the brain with the hemispheres divided.",
        exists=True,
    )
    outSegmentation = File(
        desc="2D image showing the tissue classification on the midsagittal plane",
        exists=True,
    )


class MedicAlgorithmSPECTRE2010(SEMLikeCommandLine):
    """SPECTRE 2010: Simple Paradigm for Extra-Cranial Tissue REmoval [1]_, [2]_.

    References
    ----------

    .. [1] A. Carass, M.B. Wheeler, J. Cuzzocreo, P.-L. Bazin, S.S. Bassett, and J.L. Prince,
           'A Joint Registration and Segmentation Approach to Skull Stripping',
           Fourth IEEE International Symposium on Biomedical Imaging (ISBI 2007), Arlington, VA,
           April 12-15, 2007.
    .. [2] A. Carass, J. Cuzzocreo, M.B. Wheeler, P.-L. Bazin, S.M. Resnick, and J.L. Prince,
           'Simple paradigm for extra-cerebral tissue removal: Algorithm and analysis',
           NeuroImage 56(4):1982-1992, 2011.

    """

    input_spec = MedicAlgorithmSPECTRE2010InputSpec
    output_spec = MedicAlgorithmSPECTRE2010OutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run edu.jhu.ece.iacl.plugins.segmentation.skull_strip.MedicAlgorithmSPECTRE2010 "
    _outputs_filenames = {
        "outd0": "outd0.nii",
        "outOriginal": "outOriginal.nii",
        "outMask": "outMask.nii",
        "outSplitHalves": "outSplitHalves.nii",
        "outMidsagittal": "outMidsagittal.nii",
        "outPrior": "outPrior.nii",
        "outFANTASM": "outFANTASM.nii",
        "outSegmentation": "outSegmentation.nii",
        "outStripped": "outStripped.nii",
    }
    _redirect_x = True


class JistBrainPartialVolumeFilterInputSpec(CommandLineInputSpec):
    inInput = File(desc="Input Image", exists=True, argstr="--inInput %s")
    inPV = traits.Enum(
        "bright",
        "dark",
        "both",
        desc="Outputs the raw intensity values or a probability score for the partial volume regions.",
        argstr="--inPV %s",
    )
    inoutput = traits.Enum(
        "probability", "intensity", desc="output", argstr="--inoutput %s"
    )
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outPartial = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Partial Volume Image",
        argstr="--outPartial %s",
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class JistBrainPartialVolumeFilterOutputSpec(TraitedSpec):
    outPartial = File(desc="Partial Volume Image", exists=True)


class JistBrainPartialVolumeFilter(SEMLikeCommandLine):
    """Partial Volume Filter.

    Filters an image for regions of partial voluming assuming a ridge-like model of intensity.

    """

    input_spec = JistBrainPartialVolumeFilterInputSpec
    output_spec = JistBrainPartialVolumeFilterOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.brain.JistBrainPartialVolumeFilter "
    _outputs_filenames = {"outPartial": "outPartial.nii"}
    _redirect_x = True


class JistIntensityMp2rageMaskingInputSpec(CommandLineInputSpec):
    inSecond = File(
        desc="Second inversion (Inv2) Image", exists=True, argstr="--inSecond %s"
    )
    inQuantitative = File(
        desc="Quantitative T1 Map (T1_Images) Image",
        exists=True,
        argstr="--inQuantitative %s",
    )
    inT1weighted = File(
        desc="T1-weighted (UNI) Image", exists=True, argstr="--inT1weighted %s"
    )
    inBackground = traits.Enum(
        "exponential",
        "half-normal",
        desc="Model distribution for background noise (default is half-normal, exponential is more stringent).",
        argstr="--inBackground %s",
    )
    inSkip = traits.Enum("true", "false", desc="Skip zero values", argstr="--inSkip %s")
    inMasking = traits.Enum(
        "binary",
        "proba",
        desc="Whether to use a binary threshold or a weighted average based on the probability.",
        argstr="--inMasking %s",
    )
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outSignal = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Signal Proba Image",
        argstr="--outSignal_Proba %s",
    )
    outSignal2 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Signal Mask Image",
        argstr="--outSignal_Mask %s",
    )
    outMasked = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Masked T1 Map Image",
        argstr="--outMasked_T1_Map %s",
    )
    outMasked2 = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Masked Iso Image",
        argstr="--outMasked_T1weighted %s",
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class JistIntensityMp2rageMaskingOutputSpec(TraitedSpec):
    outSignal = File(desc="Signal Proba Image", exists=True)
    outSignal2 = File(desc="Signal Mask Image", exists=True)
    outMasked = File(desc="Masked T1 Map Image", exists=True)
    outMasked2 = File(desc="Masked Iso Image", exists=True)


class JistIntensityMp2rageMasking(SEMLikeCommandLine):
    """Estimate a background signal mask for a MP2RAGE dataset."""

    input_spec = JistIntensityMp2rageMaskingInputSpec
    output_spec = JistIntensityMp2rageMaskingOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run de.mpg.cbs.jist.intensity.JistIntensityMp2rageMasking "
    _outputs_filenames = {
        "outSignal2": "outSignal2.nii",
        "outSignal": "outSignal.nii",
        "outMasked2": "outMasked2.nii",
        "outMasked": "outMasked.nii",
    }
    _redirect_x = True


class MedicAlgorithmThresholdToBinaryMaskInputSpec(CommandLineInputSpec):
    inLabel = InputMultiPath(File, desc="Input volumes", sep=";", argstr="--inLabel %s")
    inMinimum = traits.Float(desc="Minimum threshold value.", argstr="--inMinimum %f")
    inMaximum = traits.Float(desc="Maximum threshold value.", argstr="--inMaximum %f")
    inUse = traits.Enum(
        "true",
        "false",
        desc="Use the images max intensity as the max value of the range.",
        argstr="--inUse %s",
    )
    xPrefExt = traits.Enum("nrrd", desc="Output File Type", argstr="--xPrefExt %s")
    outBinary = InputMultiPath(
        File, desc="Binary Mask", sep=";", argstr="--outBinary %s"
    )
    null = traits.Str(desc="Execution Time", argstr="--null %s")
    xDefaultMem = traits.Int(
        desc="Set default maximum heap size", argstr="-xDefaultMem %d"
    )
    xMaxProcess = traits.Int(
        1,
        desc="Set default maximum number of processes.",
        argstr="-xMaxProcess %d",
        usedefault=True,
    )


class MedicAlgorithmThresholdToBinaryMaskOutputSpec(TraitedSpec):
    pass


class MedicAlgorithmThresholdToBinaryMask(SEMLikeCommandLine):
    """Threshold to Binary Mask.

    Given a volume and an intensity range create a binary mask for values within that range.

    """

    input_spec = MedicAlgorithmThresholdToBinaryMaskInputSpec
    output_spec = MedicAlgorithmThresholdToBinaryMaskOutputSpec
    _cmd = "java edu.jhu.ece.iacl.jist.cli.run edu.jhu.ece.iacl.plugins.utilities.volume.MedicAlgorithmThresholdToBinaryMask "
    _outputs_filenames = {}
    _redirect_x = True



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..calib import SFLUTGen


def test_SFLUTGen_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        binincsize=dict(
            argstr="-binincsize %d",
            units="NA",
        ),
        directmap=dict(
            argstr="-directmap",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="-inputfile %s",
            extensions=None,
            mandatory=True,
        ),
        info_file=dict(
            argstr="-infofile %s",
            extensions=None,
            mandatory=True,
        ),
        minvectsperbin=dict(
            argstr="-minvectsperbin %d",
            units="NA",
        ),
        order=dict(
            argstr="-order %d",
            units="NA",
        ),
        out_file=dict(
            argstr="> %s",
            extensions=None,
            genfile=True,
            position=-1,
        ),
        outputstem=dict(
            argstr="-outputstem %s",
            usedefault=True,
        ),
        pdf=dict(
            argstr="-pdf %s",
            usedefault=True,
        ),
    )
    inputs = SFLUTGen.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_SFLUTGen_outputs():
    output_map = dict(
        lut_one_fibre=dict(
            extensions=None,
        ),
        lut_two_fibres=dict(
            extensions=None,
        ),
    )
    outputs = SFLUTGen.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import AvScale


def test_AvScale_inputs():
    input_map = dict(
        all_param=dict(
            argstr="--allparams",
        ),
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        mat_file=dict(
            argstr="%s",
            extensions=None,
            position=-2,
        ),
        ref_file=dict(
            argstr="%s",
            extensions=None,
            position=-1,
        ),
    )
    inputs = AvScale.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_AvScale_outputs():
    output_map = dict(
        average_scaling=dict(),
        backward_half_transform=dict(),
        determinant=dict(),
        forward_half_transform=dict(),
        left_right_orientation_preserved=dict(),
        rot_angles=dict(),
        rotation_translation_matrix=dict(),
        scales=dict(),
        skews=dict(),
        translations=dict(),
    )
    outputs = AvScale.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""The regutils module provides classes for interfacing with the `niftyreg
<http://niftyreg.sourceforge.net>`_ utility command line tools.

The interfaces were written to work with niftyreg version 1.5.10
"""
import os

from ..base import TraitedSpec, File, traits, isdefined
from .base import get_custom_path, NiftyRegCommand, NiftyRegCommandInputSpec
from ...utils.filemanip import split_filename


class RegResampleInputSpec(NiftyRegCommandInputSpec):
    """ Input Spec for RegResample. """

    # Input reference file
    ref_file = File(
        exists=True,
        desc="The input reference/target image",
        argstr="-ref %s",
        mandatory=True,
    )
    # Input floating file
    flo_file = File(
        exists=True,
        desc="The input floating/source image",
        argstr="-flo %s",
        mandatory=True,
    )
    # Input deformation field
    trans_file = File(
        exists=True, desc="The input transformation file", argstr="-trans %s"
    )

    type = traits.Enum(
        "res",
        "blank",
        argstr="-%s",
        position=-2,
        usedefault=True,
        desc="Type of output",
    )

    # Output file name
    out_file = File(
        name_source=["flo_file"],
        name_template="%s",
        argstr="%s",
        position=-1,
        desc="The output filename of the transformed image",
    )

    # Interpolation type
    inter_val = traits.Enum(
        "NN", "LIN", "CUB", "SINC", desc="Interpolation type", argstr="-inter %d"
    )

    # Padding value
    pad_val = traits.Float(desc="Padding value", argstr="-pad %f")

    # Tensor flag
    tensor_flag = traits.Bool(desc="Resample Tensor Map", argstr="-tensor ")

    # Verbosity off
    verbosity_off_flag = traits.Bool(argstr="-voff", desc="Turn off verbose output")
    # PSF flag
    desc = "Perform the resampling in two steps to resample an image to a \
lower resolution"

    psf_flag = traits.Bool(argstr="-psf", desc=desc)
    desc = "Minimise the matrix metric (0) or the determinant (1) when \
estimating the PSF [0]"

    psf_alg = traits.Enum(0, 1, argstr="-psf_alg %d", desc=desc)


class RegResampleOutputSpec(TraitedSpec):
    """ Output Spec for RegResample. """

    out_file = File(desc="The output filename of the transformed image")


class RegResample(NiftyRegCommand):
    """Interface for executable reg_resample from NiftyReg platform.

    Tool to resample floating image in the space of a defined reference image
    given a transformation parametrisation generated by reg_aladin, reg_f3d or
    reg_transform

    `Source code <https://cmiclab.cs.ucl.ac.uk/mmodat/niftyreg>`_

    Examples
    --------
    >>> from nipype.interfaces import niftyreg
    >>> node = niftyreg.RegResample()
    >>> node.inputs.ref_file = 'im1.nii'
    >>> node.inputs.flo_file = 'im2.nii'
    >>> node.inputs.trans_file = 'warpfield.nii'
    >>> node.inputs.inter_val = 'LIN'
    >>> node.inputs.omp_core_val = 4
    >>> node.cmdline
    'reg_resample -flo im2.nii -inter 1 -omp 4 -ref im1.nii -trans \
warpfield.nii -res im2_res.nii.gz'

    """

    _cmd = get_custom_path("reg_resample")
    input_spec = RegResampleInputSpec
    output_spec = RegResampleOutputSpec

    # Need this overload to properly constraint the interpolation type input
    def _format_arg(self, name, spec, value):
        if name == "inter_val":
            inter_val = {"NN": 0, "LIN": 1, "CUB": 3, "SINC": 4}
            return spec.argstr % inter_val[value]
        else:
            return super(RegResample, self)._format_arg(name, spec, value)

    def _overload_extension(self, value, name=None):
        path, base, _ = split_filename(value)
        suffix = self.inputs.type
        return os.path.join(path, "{0}_{1}.nii.gz".format(base, suffix))


class RegJacobianInputSpec(NiftyRegCommandInputSpec):
    """ Input Spec for RegJacobian. """

    # Reference file name
    desc = "Reference/target file (required if specifying CPP transformations."
    ref_file = File(exists=True, desc=desc, argstr="-ref %s")
    # Input transformation file
    trans_file = File(
        exists=True,
        desc="The input non-rigid transformation",
        argstr="-trans %s",
        mandatory=True,
    )
    type = traits.Enum(
        "jac",
        "jacL",
        "jacM",
        usedefault=True,
        argstr="-%s",
        position=-2,
        desc="Type of jacobian outcome",
    )
    out_file = File(
        name_source=["trans_file"],
        name_template="%s",
        desc="The output jacobian determinant file name",
        argstr="%s",
        position=-1,
    )


class RegJacobianOutputSpec(TraitedSpec):
    """ Output Spec for RegJacobian. """

    out_file = File(desc="The output file")


class RegJacobian(NiftyRegCommand):
    """Interface for executable reg_resample from NiftyReg platform.

    Tool to generate Jacobian determinant maps from transformation
    parametrisation generated by reg_f3d

    `Source code <https://cmiclab.cs.ucl.ac.uk/mmodat/niftyreg>`_

    Examples
    --------
    >>> from nipype.interfaces import niftyreg
    >>> node = niftyreg.RegJacobian()
    >>> node.inputs.ref_file = 'im1.nii'
    >>> node.inputs.trans_file = 'warpfield.nii'
    >>> node.inputs.omp_core_val = 4
    >>> node.cmdline
    'reg_jacobian -omp 4 -ref im1.nii -trans warpfield.nii -jac \
warpfield_jac.nii.gz'

    """

    _cmd = get_custom_path("reg_jacobian")
    input_spec = RegJacobianInputSpec
    output_spec = RegJacobianOutputSpec

    def _overload_extension(self, value, name=None):
        path, base, _ = split_filename(value)
        suffix = self.inputs.type
        return os.path.join(path, "{0}_{1}.nii.gz".format(base, suffix))


class RegToolsInputSpec(NiftyRegCommandInputSpec):
    """ Input Spec for RegTools. """

    # Input image file
    in_file = File(
        exists=True, desc="The input image file path", argstr="-in %s", mandatory=True
    )

    # Output file path
    out_file = File(
        name_source=["in_file"],
        name_template="%s_tools.nii.gz",
        desc="The output file name",
        argstr="-out %s",
    )

    # Make the output image isotropic
    iso_flag = traits.Bool(argstr="-iso", desc="Make output image isotropic")

    # Set scale, slope to 0 and 1.
    noscl_flag = traits.Bool(argstr="-noscl", desc="Set scale, slope to 0 and 1")

    # Values outside the mask are set to NaN
    mask_file = File(
        exists=True, desc="Values outside the mask are set to NaN", argstr="-nan %s"
    )

    # Threshold the input image
    desc = "Binarise the input image with the given threshold"
    thr_val = traits.Float(desc=desc, argstr="-thr %f")

    # Binarise the input image
    bin_flag = traits.Bool(argstr="-bin", desc="Binarise the input image")

    # Compute the mean RMS between the two images
    rms_val = File(
        exists=True, desc="Compute the mean RMS between the images", argstr="-rms %s"
    )

    # Perform division by image or value
    div_val = traits.Either(
        traits.Float,
        File(exists=True),
        desc="Divide the input by image or value",
        argstr="-div %s",
    )

    # Perform multiplication by image or value
    mul_val = traits.Either(
        traits.Float,
        File(exists=True),
        desc="Multiply the input by image or value",
        argstr="-mul %s",
    )

    # Perform addition by image or value
    add_val = traits.Either(
        traits.Float,
        File(exists=True),
        desc="Add to the input image or value",
        argstr="-add %s",
    )

    # Perform subtraction by image or value
    sub_val = traits.Either(
        traits.Float,
        File(exists=True),
        desc="Add to the input image or value",
        argstr="-sub %s",
    )

    # Downsample the image by a factor of 2.
    down_flag = traits.Bool(
        desc="Downsample the image by a factor of 2", argstr="-down"
    )

    # Smoothing using spline kernel
    desc = "Smooth the input image using a cubic spline kernel"
    smo_s_val = traits.Tuple(
        traits.Float, traits.Float, traits.Float, desc=desc, argstr="-smoS %f %f %f"
    )

    # Change the resolution of the input image
    chg_res_val = traits.Tuple(
        traits.Float,
        traits.Float,
        traits.Float,
        desc="Change the resolution of the input image",
        argstr="-chgres %f %f %f",
    )

    # Smoothing using Gaussian kernel
    desc = "Smooth the input image using a Gaussian kernel"
    smo_g_val = traits.Tuple(
        traits.Float, traits.Float, traits.Float, desc=desc, argstr="-smoG %f %f %f"
    )

    # Interpolation type
    inter_val = traits.Enum(
        "NN",
        "LIN",
        "CUB",
        "SINC",
        desc="Interpolation order to use to warp the floating image",
        argstr="-interp %d",
    )


class RegToolsOutputSpec(TraitedSpec):
    """ Output Spec for RegTools. """

    out_file = File(desc="The output file", exists=True)


class RegTools(NiftyRegCommand):
    """Interface for executable reg_tools from NiftyReg platform.

    Tool delivering various actions related to registration such as
    resampling the input image to a chosen resolution or remove the nan and
    inf in the input image by a specified value.

    `Source code <https://cmiclab.cs.ucl.ac.uk/mmodat/niftyreg>`_

    Examples
    --------
    >>> from nipype.interfaces import niftyreg
    >>> node = niftyreg.RegTools()
    >>> node.inputs.in_file = 'im1.nii'
    >>> node.inputs.mul_val = 4
    >>> node.inputs.omp_core_val = 4
    >>> node.cmdline
    'reg_tools -in im1.nii -mul 4.0 -omp 4 -out im1_tools.nii.gz'

    """

    _cmd = get_custom_path("reg_tools")
    input_spec = RegToolsInputSpec
    output_spec = RegToolsOutputSpec
    _suffix = "_tools"

    # Need this overload to properly constraint the interpolation type input
    def _format_arg(self, name, spec, value):
        if name == "inter_val":
            inter_val = {"NN": 0, "LIN": 1, "CUB": 3, "SINC": 4}
            return spec.argstr % inter_val[value]
        else:
            return super(RegTools, self)._format_arg(name, spec, value)


class RegAverageInputSpec(NiftyRegCommandInputSpec):
    """ Input Spec for RegAverage. """

    avg_files = traits.List(
        File(exist=True),
        position=1,
        argstr="-avg %s",
        sep=" ",
        xor=[
            "avg_lts_files",
            "avg_ref_file",
            "demean1_ref_file",
            "demean2_ref_file",
            "demean3_ref_file",
            "warp_files",
        ],
        desc="Averaging of images/affine transformations",
    )

    desc = "Robust average of affine transformations"
    avg_lts_files = traits.List(
        File(exist=True),
        position=1,
        argstr="-avg_lts %s",
        sep=" ",
        xor=[
            "avg_files",
            "avg_ref_file",
            "demean1_ref_file",
            "demean2_ref_file",
            "demean3_ref_file",
            "warp_files",
        ],
        desc=desc,
    )

    desc = "All input images are resampled into the space of <reference image>\
 and averaged. A cubic spline interpolation scheme is used for resampling"

    avg_ref_file = File(
        position=1,
        argstr="-avg_tran %s",
        xor=[
            "avg_files",
            "avg_lts_files",
            "demean1_ref_file",
            "demean2_ref_file",
            "demean3_ref_file",
        ],
        requires=["warp_files"],
        desc=desc,
    )

    desc = "Average images and demean average image that have affine \
transformations to a common space"

    demean1_ref_file = File(
        position=1,
        argstr="-demean1 %s",
        xor=[
            "avg_files",
            "avg_lts_files",
            "avg_ref_file",
            "demean2_ref_file",
            "demean3_ref_file",
        ],
        requires=["warp_files"],
        desc=desc,
    )

    desc = "Average images and demean average image that have non-rigid \
transformations to a common space"

    demean2_ref_file = File(
        position=1,
        argstr="-demean2 %s",
        xor=[
            "avg_files",
            "avg_lts_files",
            "avg_ref_file",
            "demean1_ref_file",
            "demean3_ref_file",
        ],
        requires=["warp_files"],
        desc=desc,
    )

    desc = "Average images and demean average image that have linear and \
non-rigid transformations to a common space"

    demean3_ref_file = File(
        position=1,
        argstr="-demean3 %s",
        xor=[
            "avg_files",
            "avg_lts_files",
            "avg_ref_file",
            "demean1_ref_file",
            "demean2_ref_file",
        ],
        requires=["warp_files"],
        desc=desc,
    )

    desc = "transformation files and floating image pairs/triplets to the \
reference space"

    warp_files = traits.List(
        File(exist=True),
        position=-1,
        argstr="%s",
        sep=" ",
        xor=["avg_files", "avg_lts_files"],
        desc=desc,
    )

    out_file = File(genfile=True, position=0, desc="Output file name", argstr="%s")


class RegAverageOutputSpec(TraitedSpec):
    """ Output Spec for RegAverage. """

    out_file = File(desc="Output file name")


class RegAverage(NiftyRegCommand):
    """Interface for executable reg_average from NiftyReg platform.

    Compute average matrix or image from a list of matrices or image.
    The tool can be use to resample images given input transformation
    parametrisation as well as to demean transformations in Euclidean or
    log-Euclidean space.

    This interface is different than the others in the way that the options
    will be written in a command file that is given as a parameter.

    `Source code <https://cmiclab.cs.ucl.ac.uk/mmodat/niftyreg>`_

    Examples
    --------

    >>> from nipype.interfaces import niftyreg
    >>> node = niftyreg.RegAverage()
    >>> one_file = 'im1.nii'
    >>> two_file = 'im2.nii'
    >>> three_file = 'im3.nii'
    >>> node.inputs.avg_files = [one_file, two_file, three_file]
    >>> node.cmdline  # doctest: +ELLIPSIS
    'reg_average --cmd_file .../reg_average_cmd'
    """

    _cmd = get_custom_path("reg_average")
    input_spec = RegAverageInputSpec
    output_spec = RegAverageOutputSpec
    _suffix = "avg_out"

    def _gen_filename(self, name):
        if name == "out_file":
            if isdefined(self.inputs.avg_lts_files):
                return self._gen_fname(self._suffix, ext=".txt")
            elif isdefined(self.inputs.avg_files):
                _, _, _ext = split_filename(self.inputs.avg_files[0])
                if _ext not in [".nii", ".nii.gz", ".hdr", ".img", ".img.gz"]:
                    return self._gen_fname(self._suffix, ext=_ext)
            return self._gen_fname(self._suffix, ext=".nii.gz")

        return None

    def _list_outputs(self):
        outputs = self.output_spec().get()

        if isdefined(self.inputs.out_file):
            outputs["out_file"] = self.inputs.out_file
        else:
            outputs["out_file"] = self._gen_filename("out_file")

        return outputs

    @property
    def cmdline(self):
        """ Rewrite the cmdline to write options in text_file."""
        argv = super(RegAverage, self).cmdline
        reg_average_cmd = os.path.join(os.getcwd(), "reg_average_cmd")
        with open(reg_average_cmd, "w") as f:
            f.write(argv)
        return "%s --cmd_file %s" % (self.cmd, reg_average_cmd)


class RegTransformInputSpec(NiftyRegCommandInputSpec):
    """ Input Spec for RegTransform. """

    ref1_file = File(
        exists=True,
        desc="The input reference/target image",
        argstr="-ref %s",
        position=0,
    )

    ref2_file = File(
        exists=True,
        desc="The input second reference/target image",
        argstr="-ref2 %s",
        position=1,
        requires=["ref1_file"],
    )

    def_input = File(
        exists=True,
        argstr="-def %s",
        position=-2,
        desc="Compute deformation field from transformation",
        xor=[
            "disp_input",
            "flow_input",
            "comp_input",
            "upd_s_form_input",
            "inv_aff_input",
            "inv_nrr_input",
            "half_input",
            "make_aff_input",
            "aff_2_rig_input",
            "flirt_2_nr_input",
        ],
    )

    disp_input = File(
        exists=True,
        argstr="-disp %s",
        position=-2,
        desc="Compute displacement field from transformation",
        xor=[
            "def_input",
            "flow_input",
            "comp_input",
            "upd_s_form_input",
            "inv_aff_input",
            "inv_nrr_input",
            "half_input",
            "make_aff_input",
            "aff_2_rig_input",
            "flirt_2_nr_input",
        ],
    )

    flow_input = File(
        exists=True,
        argstr="-flow %s",
        position=-2,
        desc="Compute flow field from spline SVF",
        xor=[
            "def_input",
            "disp_input",
            "comp_input",
            "upd_s_form_input",
            "inv_aff_input",
            "inv_nrr_input",
            "half_input",
            "make_aff_input",
            "aff_2_rig_input",
            "flirt_2_nr_input",
        ],
    )

    comp_input = File(
        exists=True,
        argstr="-comp %s",
        position=-3,
        desc="compose two transformations",
        xor=[
            "def_input",
            "disp_input",
            "flow_input",
            "upd_s_form_input",
            "inv_aff_input",
            "inv_nrr_input",
            "half_input",
            "make_aff_input",
            "aff_2_rig_input",
            "flirt_2_nr_input",
        ],
        requires=["comp_input2"],
    )

    comp_input2 = File(
        exists=True, argstr="%s", position=-2, desc="compose two transformations"
    )

    desc = "Update s-form using the affine transformation"
    upd_s_form_input = File(
        exists=True,
        argstr="-updSform %s",
        position=-3,
        desc=desc,
        xor=[
            "def_input",
            "disp_input",
            "flow_input",
            "comp_input",
            "inv_aff_input",
            "inv_nrr_input",
            "half_input",
            "make_aff_input",
            "aff_2_rig_input",
            "flirt_2_nr_input",
        ],
        requires=["upd_s_form_input2"],
    )

    desc = "Update s-form using the affine transformation"
    upd_s_form_input2 = File(
        exists=True, argstr="%s", position=-2, desc=desc, requires=["upd_s_form_input"]
    )

    inv_aff_input = File(
        exists=True,
        argstr="-invAff %s",
        position=-2,
        desc="Invert an affine transformation",
        xor=[
            "def_input",
            "disp_input",
            "flow_input",
            "comp_input",
            "upd_s_form_input",
            "inv_nrr_input",
            "half_input",
            "make_aff_input",
            "aff_2_rig_input",
            "flirt_2_nr_input",
        ],
    )

    inv_nrr_input = traits.Tuple(
        File(exists=True),
        File(exists=True),
        desc="Invert a non-linear transformation",
        argstr="-invNrr %s %s",
        position=-2,
        xor=[
            "def_input",
            "disp_input",
            "flow_input",
            "comp_input",
            "upd_s_form_input",
            "inv_aff_input",
            "half_input",
            "make_aff_input",
            "aff_2_rig_input",
            "flirt_2_nr_input",
        ],
    )

    half_input = File(
        exists=True,
        argstr="-half %s",
        position=-2,
        desc="Half way to the input transformation",
        xor=[
            "def_input",
            "disp_input",
            "flow_input",
            "comp_input",
            "upd_s_form_input",
            "inv_aff_input",
            "inv_nrr_input",
            "make_aff_input",
            "aff_2_rig_input",
            "flirt_2_nr_input",
        ],
    )

    argstr_tmp = "-makeAff %f %f %f %f %f %f %f %f %f %f %f %f"
    make_aff_input = traits.Tuple(
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        traits.Float,
        argstr=argstr_tmp,
        position=-2,
        desc="Make an affine transformation matrix",
        xor=[
            "def_input",
            "disp_input",
            "flow_input",
            "comp_input",
            "upd_s_form_input",
            "inv_aff_input",
            "inv_nrr_input",
            "half_input",
            "aff_2_rig_input",
            "flirt_2_nr_input",
        ],
    )

    desc = "Extract the rigid component from affine transformation"
    aff_2_rig_input = File(
        exists=True,
        argstr="-aff2rig %s",
        position=-2,
        desc=desc,
        xor=[
            "def_input",
            "disp_input",
            "flow_input",
            "comp_input",
            "upd_s_form_input",
            "inv_aff_input",
            "inv_nrr_input",
            "half_input",
            "make_aff_input",
            "flirt_2_nr_input",
        ],
    )

    desc = "Convert a FLIRT affine transformation to niftyreg affine \
transformation"

    flirt_2_nr_input = traits.Tuple(
        File(exists=True),
        File(exists=True),
        File(exists=True),
        argstr="-flirtAff2NR %s %s %s",
        position=-2,
        desc=desc,
        xor=[
            "def_input",
            "disp_input",
            "flow_input",
            "comp_input",
            "upd_s_form_input",
            "inv_aff_input",
            "inv_nrr_input",
            "half_input",
            "make_aff_input",
            "aff_2_rig_input",
        ],
    )

    out_file = File(
        genfile=True, position=-1, argstr="%s", desc="transformation file to write"
    )


class RegTransformOutputSpec(TraitedSpec):
    """ Output Spec for RegTransform. """

    out_file = File(desc="Output File (transformation in any format)")


class RegTransform(NiftyRegCommand):
    """Interface for executable reg_transform from NiftyReg platform.

    Tools to convert transformation parametrisation from one type to another
    as well as to compose, inverse or half transformations.

    `Source code <https://cmiclab.cs.ucl.ac.uk/mmodat/niftyreg>`_

    Examples
    --------
    >>> from nipype.interfaces import niftyreg
    >>> node = niftyreg.RegTransform()
    >>> node.inputs.def_input = 'warpfield.nii'
    >>> node.inputs.omp_core_val = 4
    >>> node.cmdline  # doctest: +ELLIPSIS
    'reg_transform -omp 4 -def warpfield.nii .../warpfield_trans.nii.gz'

    """

    _cmd = get_custom_path("reg_transform")
    input_spec = RegTransformInputSpec
    output_spec = RegTransformOutputSpec
    _suffix = "_trans"

    def _find_input(self):
        inputs = [
            self.inputs.def_input,
            self.inputs.disp_input,
            self.inputs.flow_input,
            self.inputs.comp_input,
            self.inputs.comp_input2,
            self.inputs.upd_s_form_input,
            self.inputs.inv_aff_input,
            self.inputs.inv_nrr_input,
            self.inputs.half_input,
            self.inputs.make_aff_input,
            self.inputs.aff_2_rig_input,
            self.inputs.flirt_2_nr_input,
        ]
        entries = []
        for entry in inputs:
            if isdefined(entry):
                entries.append(entry)
                _, _, ext = split_filename(entry)
                if ext == ".nii" or ext == ".nii.gz" or ext == ".hdr":
                    return entry
        if len(entries):
            return entries[0]
        return None

    def _gen_filename(self, name):
        if name == "out_file":
            if isdefined(self.inputs.make_aff_input):
                return self._gen_fname("matrix", suffix=self._suffix, ext=".txt")

            if isdefined(self.inputs.comp_input) and isdefined(self.inputs.comp_input2):
                _, bn1, ext1 = split_filename(self.inputs.comp_input)
                _, _, ext2 = split_filename(self.inputs.comp_input2)
                if ext1 in [".nii", ".nii.gz", ".hdr", ".img", ".img.gz"] or ext2 in [
                    ".nii",
                    ".nii.gz",
                    ".hdr",
                    ".img",
                    ".img.gz",
                ]:
                    return self._gen_fname(bn1, suffix=self._suffix, ext=".nii.gz")
                else:
                    return self._gen_fname(bn1, suffix=self._suffix, ext=ext1)

            if isdefined(self.inputs.flirt_2_nr_input):
                return self._gen_fname(
                    self.inputs.flirt_2_nr_input[0], suffix=self._suffix, ext=".txt"
                )

            input_to_use = self._find_input()
            _, _, ext = split_filename(input_to_use)
            if ext not in [".nii", ".nii.gz", ".hdr", ".img", ".img.gz"]:
                return self._gen_fname(input_to_use, suffix=self._suffix, ext=ext)
            else:
                return self._gen_fname(input_to_use, suffix=self._suffix, ext=".nii.gz")

        return None

    def _list_outputs(self):
        outputs = self.output_spec().get()

        if isdefined(self.inputs.out_file):
            outputs["out_file"] = self.inputs.out_file
        else:
            outputs["out_file"] = self._gen_filename("out_file")

        return outputs


class RegMeasureInputSpec(NiftyRegCommandInputSpec):
    """ Input Spec for RegMeasure. """

    # Input reference file
    ref_file = File(
        exists=True,
        desc="The input reference/target image",
        argstr="-ref %s",
        mandatory=True,
    )
    # Input floating file
    flo_file = File(
        exists=True,
        desc="The input floating/source image",
        argstr="-flo %s",
        mandatory=True,
    )
    measure_type = traits.Enum(
        "ncc",
        "lncc",
        "nmi",
        "ssd",
        mandatory=True,
        argstr="-%s",
        desc="Measure of similarity to compute",
    )
    out_file = File(
        name_source=["flo_file"],
        name_template="%s",
        argstr="-out %s",
        desc="The output text file containing the measure",
    )


class RegMeasureOutputSpec(TraitedSpec):
    """ Output Spec for RegMeasure. """

    out_file = File(desc="The output text file containing the measure")


class RegMeasure(NiftyRegCommand):
    """Interface for executable reg_measure from NiftyReg platform.

    Given two input images, compute the specified measure(s) of similarity

    `Source code <https://cmiclab.cs.ucl.ac.uk/mmodat/niftyreg>`_

    Examples
    --------
    >>> from nipype.interfaces import niftyreg
    >>> node = niftyreg.RegMeasure()
    >>> node.inputs.ref_file = 'im1.nii'
    >>> node.inputs.flo_file = 'im2.nii'
    >>> node.inputs.measure_type = 'lncc'
    >>> node.inputs.omp_core_val = 4
    >>> node.cmdline
    'reg_measure -flo im2.nii -lncc -omp 4 -out im2_lncc.txt -ref im1.nii'

    """

    _cmd = get_custom_path("reg_measure")
    input_spec = RegMeasureInputSpec
    output_spec = RegMeasureOutputSpec

    def _overload_extension(self, value, name=None):
        path, base, _ = split_filename(value)
        suffix = self.inputs.measure_type
        return os.path.join(path, "{0}_{1}.txt".format(base, suffix))



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..label_fusion import LabelFusion


def test_LabelFusion_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        classifier_type=dict(
            argstr="-%s",
            mandatory=True,
            position=2,
        ),
        conv=dict(
            argstr="-conv %f",
        ),
        dilation_roi=dict(),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        file_to_seg=dict(
            extensions=None,
            mandatory=True,
        ),
        in_file=dict(
            argstr="-in %s",
            extensions=None,
            mandatory=True,
            position=1,
        ),
        kernel_size=dict(),
        mask_file=dict(
            argstr="-mask %s",
            extensions=None,
        ),
        max_iter=dict(
            argstr="-max_iter %d",
        ),
        mrf_value=dict(
            argstr="-MRF_beta %f",
        ),
        out_file=dict(
            argstr="-out %s",
            extensions=None,
            name_source=["in_file"],
            name_template="%s",
        ),
        prob_flag=dict(
            argstr="-outProb",
        ),
        prob_update_flag=dict(
            argstr="-prop_update",
        ),
        proportion=dict(
            argstr="-prop %s",
        ),
        set_pq=dict(
            argstr="-setPQ %f %f",
        ),
        sm_ranking=dict(
            argstr="-%s",
            position=3,
            usedefault=True,
        ),
        template_file=dict(
            extensions=None,
        ),
        template_num=dict(),
        unc=dict(
            argstr="-unc",
        ),
        unc_thresh=dict(
            argstr="-uncthres %f",
        ),
        verbose=dict(
            argstr="-v %s",
        ),
    )
    inputs = LabelFusion.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_LabelFusion_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = LabelFusion.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..brains import BRAINSMush


def test_BRAINSMush_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        boundingBoxSize=dict(
            argstr="--boundingBoxSize %s",
            sep=",",
        ),
        boundingBoxStart=dict(
            argstr="--boundingBoxStart %s",
            sep=",",
        ),
        desiredMean=dict(
            argstr="--desiredMean %f",
        ),
        desiredVariance=dict(
            argstr="--desiredVariance %f",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputFirstVolume=dict(
            argstr="--inputFirstVolume %s",
            extensions=None,
        ),
        inputMaskVolume=dict(
            argstr="--inputMaskVolume %s",
            extensions=None,
        ),
        inputSecondVolume=dict(
            argstr="--inputSecondVolume %s",
            extensions=None,
        ),
        lowerThresholdFactor=dict(
            argstr="--lowerThresholdFactor %f",
        ),
        lowerThresholdFactorPre=dict(
            argstr="--lowerThresholdFactorPre %f",
        ),
        numberOfThreads=dict(
            argstr="--numberOfThreads %d",
        ),
        outputMask=dict(
            argstr="--outputMask %s",
            hash_files=False,
        ),
        outputVolume=dict(
            argstr="--outputVolume %s",
            hash_files=False,
        ),
        outputWeightsFile=dict(
            argstr="--outputWeightsFile %s",
            hash_files=False,
        ),
        seed=dict(
            argstr="--seed %s",
            sep=",",
        ),
        upperThresholdFactor=dict(
            argstr="--upperThresholdFactor %f",
        ),
        upperThresholdFactorPre=dict(
            argstr="--upperThresholdFactorPre %f",
        ),
    )
    inputs = BRAINSMush.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_BRAINSMush_outputs():
    output_map = dict(
        outputMask=dict(
            extensions=None,
        ),
        outputVolume=dict(
            extensions=None,
        ),
        outputWeightsFile=dict(
            extensions=None,
        ),
    )
    outputs = BRAINSMush.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import Eval


def test_Eval_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        expr=dict(
            argstr='-expr "%s"',
            mandatory=True,
            position=3,
        ),
        in_file_a=dict(
            argstr="-a %s",
            extensions=None,
            mandatory=True,
            position=0,
        ),
        in_file_b=dict(
            argstr="-b %s",
            extensions=None,
            position=1,
        ),
        in_file_c=dict(
            argstr="-c %s",
            extensions=None,
            position=2,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        other=dict(
            argstr="",
            extensions=None,
        ),
        out1D=dict(
            argstr="-1D",
        ),
        out_file=dict(
            argstr="-prefix %s",
            extensions=None,
            name_source="in_file_a",
            name_template="%s_calc",
        ),
        outputtype=dict(),
        single_idx=dict(),
        start_idx=dict(
            requires=["stop_idx"],
        ),
        stop_idx=dict(
            requires=["start_idx"],
        ),
    )
    inputs = Eval.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Eval_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = Eval.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..convert import ProcStreamlines


def test_ProcStreamlines_inputs():
    input_map = dict(
        allowmultitargets=dict(
            argstr="-allowmultitargets",
        ),
        args=dict(
            argstr="%s",
        ),
        datadims=dict(
            argstr="-datadims %s",
            units="voxels",
        ),
        directional=dict(
            argstr="-directional %s",
            units="NA",
        ),
        discardloops=dict(
            argstr="-discardloops",
        ),
        endpointfile=dict(
            argstr="-endpointfile %s",
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        exclusionfile=dict(
            argstr="-exclusionfile %s",
            extensions=None,
        ),
        gzip=dict(
            argstr="-gzip",
        ),
        in_file=dict(
            argstr="-inputfile %s",
            extensions=None,
            mandatory=True,
            position=1,
        ),
        inputmodel=dict(
            argstr="-inputmodel %s",
            usedefault=True,
        ),
        iterations=dict(
            argstr="-iterations %d",
            units="NA",
        ),
        maxtractlength=dict(
            argstr="-maxtractlength %d",
            units="mm",
        ),
        maxtractpoints=dict(
            argstr="-maxtractpoints %d",
            units="NA",
        ),
        mintractlength=dict(
            argstr="-mintractlength %d",
            units="mm",
        ),
        mintractpoints=dict(
            argstr="-mintractpoints %d",
            units="NA",
        ),
        noresample=dict(
            argstr="-noresample",
        ),
        out_file=dict(
            argstr="> %s",
            extensions=None,
            genfile=True,
            position=-1,
        ),
        outputacm=dict(
            argstr="-outputacm",
            requires=["outputroot", "seedfile"],
        ),
        outputcbs=dict(
            argstr="-outputcbs",
            requires=["outputroot", "targetfile", "seedfile"],
        ),
        outputcp=dict(
            argstr="-outputcp",
            requires=["outputroot", "seedfile"],
        ),
        outputroot=dict(
            argstr="-outputroot %s",
            extensions=None,
        ),
        outputsc=dict(
            argstr="-outputsc",
            requires=["outputroot", "seedfile"],
        ),
        outputtracts=dict(
            argstr="-outputtracts",
        ),
        regionindex=dict(
            argstr="-regionindex %d",
            units="mm",
        ),
        resamplestepsize=dict(
            argstr="-resamplestepsize %d",
            units="NA",
        ),
        seedfile=dict(
            argstr="-seedfile %s",
            extensions=None,
        ),
        seedpointmm=dict(
            argstr="-seedpointmm %s",
            units="mm",
        ),
        seedpointvox=dict(
            argstr="-seedpointvox %s",
            units="voxels",
        ),
        targetfile=dict(
            argstr="-targetfile %s",
            extensions=None,
        ),
        truncateinexclusion=dict(
            argstr="-truncateinexclusion",
        ),
        truncateloops=dict(
            argstr="-truncateloops",
        ),
        voxeldims=dict(
            argstr="-voxeldims %s",
            units="mm",
        ),
        waypointfile=dict(
            argstr="-waypointfile %s",
            extensions=None,
        ),
    )
    inputs = ProcStreamlines.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ProcStreamlines_outputs():
    output_map = dict(
        outputroot_files=dict(),
        proc=dict(
            extensions=None,
        ),
    )
    outputs = ProcStreamlines.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os

import numpy as np

from ...testing import utils

from .. import nilearn as iface
from ...pipeline import engine as pe

import pytest
import numpy.testing as npt

no_nilearn = True
try:
    __import__("nilearn")
    no_nilearn = False
except ImportError:
    pass


@pytest.mark.skipif(no_nilearn, reason="the nilearn library is not available")
class TestSignalExtraction:

    filenames = {
        "in_file": "fmri.nii",
        "label_files": "labels.nii",
        "4d_label_file": "4dlabels.nii",
        "out_file": "signals.tsv",
    }
    labels = ["CSF", "GrayMatter", "WhiteMatter"]
    global_labels = ["GlobalSignal"] + labels

    @pytest.fixture(autouse=True, scope="class")
    def setup_class(self, tmpdir_factory):
        tempdir = tmpdir_factory.mktemp("test")
        self.orig_dir = tempdir.chdir()
        utils.save_toy_nii(self.fake_fmri_data, self.filenames["in_file"])
        utils.save_toy_nii(self.fake_label_data, self.filenames["label_files"])

    def test_signal_extract_no_shared(self):
        # run
        iface.SignalExtraction(
            in_file=self.filenames["in_file"],
            label_files=self.filenames["label_files"],
            class_labels=self.labels,
            incl_shared_variance=False,
        ).run()
        # assert
        self.assert_expected_output(self.labels, self.base_wanted)

    def test_signal_extr_bad_label_list(self):
        # run
        with pytest.raises(ValueError):
            iface.SignalExtraction(
                in_file=self.filenames["in_file"],
                label_files=self.filenames["label_files"],
                class_labels=["bad"],
                incl_shared_variance=False,
            ).run()

    def test_signal_extr_equiv_4d_no_shared(self):
        self._test_4d_label(
            self.base_wanted, self.fake_equiv_4d_label_data, incl_shared_variance=False
        )

    def test_signal_extr_4d_no_shared(self):
        # set up & run & assert
        self._test_4d_label(
            self.fourd_wanted, self.fake_4d_label_data, incl_shared_variance=False
        )

    def test_signal_extr_global_no_shared(self):
        # set up
        wanted_global = [[-4.0 / 6], [-1.0 / 6], [3.0 / 6], [-1.0 / 6], [-7.0 / 6]]
        for i, vals in enumerate(self.base_wanted):
            wanted_global[i].extend(vals)

        # run
        iface.SignalExtraction(
            in_file=self.filenames["in_file"],
            label_files=self.filenames["label_files"],
            class_labels=self.labels,
            include_global=True,
            incl_shared_variance=False,
        ).run()

        # assert
        self.assert_expected_output(self.global_labels, wanted_global)

    def test_signal_extr_4d_global_no_shared(self):
        # set up
        wanted_global = [[3.0 / 8], [-3.0 / 8], [1.0 / 8], [-7.0 / 8], [-9.0 / 8]]
        for i, vals in enumerate(self.fourd_wanted):
            wanted_global[i].extend(vals)

        # run & assert
        self._test_4d_label(
            wanted_global,
            self.fake_4d_label_data,
            include_global=True,
            incl_shared_variance=False,
        )

    def test_signal_extr_shared(self):
        # set up
        wanted = []
        for vol in range(self.fake_fmri_data.shape[3]):
            volume = self.fake_fmri_data[:, :, :, vol].flatten()
            wanted_row = []
            for reg in range(self.fake_4d_label_data.shape[3]):
                region = self.fake_4d_label_data[:, :, :, reg].flatten()
                wanted_row.append((volume * region).sum() / (region * region).sum())

            wanted.append(wanted_row)
        # run & assert
        self._test_4d_label(wanted, self.fake_4d_label_data)

    def test_signal_extr_traits_valid(self):
        """ Test a node using the SignalExtraction interface.
        Unlike interface.run(), node.run() checks the traits
        """
        # run
        node = pe.Node(
            iface.SignalExtraction(
                in_file=os.path.abspath(self.filenames["in_file"]),
                label_files=os.path.abspath(self.filenames["label_files"]),
                class_labels=self.labels,
                incl_shared_variance=False,
            ),
            name="SignalExtraction",
        )
        node.run()

        # assert
        # just checking that it passes trait validations

    def _test_4d_label(
        self, wanted, fake_labels, include_global=False, incl_shared_variance=True
    ):
        # set up
        utils.save_toy_nii(fake_labels, self.filenames["4d_label_file"])

        # run
        iface.SignalExtraction(
            in_file=self.filenames["in_file"],
            label_files=self.filenames["4d_label_file"],
            class_labels=self.labels,
            incl_shared_variance=incl_shared_variance,
            include_global=include_global,
        ).run()

        wanted_labels = self.global_labels if include_global else self.labels

        # assert
        self.assert_expected_output(wanted_labels, wanted)

    def assert_expected_output(self, labels, wanted):
        with open(self.filenames["out_file"], "r") as output:
            got = [line.split() for line in output]
            labels_got = got.pop(0)  # remove header
            assert labels_got == labels
            assert len(got) == self.fake_fmri_data.shape[3], "num rows and num volumes"
            # convert from string to float
            got = [[float(num) for num in row] for row in got]
            for i, time in enumerate(got):
                assert len(labels) == len(time)
                for j, segment in enumerate(time):
                    npt.assert_almost_equal(segment, wanted[i][j], decimal=1)

    # dj: self doesnt have orig_dir at this point, not sure how to change it.
    # should work without it
    #    def teardown_class(self):
    #        self.orig_dir.chdir()

    fake_fmri_data = np.array(
        [
            [
                [[2, -1, 4, -2, 3], [4, -2, -5, -1, 0]],
                [[-2, 0, 1, 4, 4], [-5, 3, -3, 1, -5]],
            ],
            [
                [[2, -2, -1, -2, -5], [3, 0, 3, -5, -2]],
                [[-4, -2, -2, 1, -2], [3, 1, 4, -3, -2]],
            ],
        ]
    )

    fake_label_data = np.array([[[1, 0], [3, 1]], [[2, 0], [1, 3]]])

    fake_equiv_4d_label_data = np.array(
        [
            [[[1.0, 0.0, 0.0], [0.0, 0.0, 0.0]], [[0.0, 0.0, 1.0], [1.0, 0.0, 0.0]]],
            [[[0.0, 1.0, 0.0], [0.0, 0.0, 0.0]], [[1.0, 0.0, 0.0], [0.0, 0.0, 1.0]]],
        ]
    )

    base_wanted = [
        [-2.33333, 2, 0.5],
        [0, -2, 0.5],
        [-0.3333333, -1, 2.5],
        [0, -2, 0.5],
        [-1.3333333, -5, 1],
    ]

    fake_4d_label_data = np.array(
        [
            [[[0.2, 0.3, 0.5], [0.1, 0.1, 0.8]], [[0.1, 0.3, 0.6], [0.3, 0.4, 0.3]]],
            [[[0.2, 0.2, 0.6], [0.0, 0.3, 0.7]], [[0.3, 0.3, 0.4], [0.3, 0.4, 0.3]]],
        ]
    )

    fourd_wanted = [
        [-5.0652173913, -5.44565217391, 5.50543478261],
        [-7.02173913043, 11.1847826087, -4.33152173913],
        [-19.0869565217, 21.2391304348, -4.57608695652],
        [5.19565217391, -3.66304347826, -1.51630434783],
        [-12.0, 3.0, 0.5],
    ]



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..converters import DicomToNrrdConverter


def test_DicomToNrrdConverter_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputDicomDirectory=dict(
            argstr="--inputDicomDirectory %s",
        ),
        outputDirectory=dict(
            argstr="--outputDirectory %s",
            hash_files=False,
        ),
        outputVolume=dict(
            argstr="--outputVolume %s",
        ),
        smallGradientThreshold=dict(
            argstr="--smallGradientThreshold %f",
        ),
        useBMatrixGradientDirections=dict(
            argstr="--useBMatrixGradientDirections ",
        ),
        useIdentityMeaseurementFrame=dict(
            argstr="--useIdentityMeaseurementFrame ",
        ),
        writeProtocolGradientsFile=dict(
            argstr="--writeProtocolGradientsFile ",
        ),
    )
    inputs = DicomToNrrdConverter.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_DicomToNrrdConverter_outputs():
    output_map = dict(
        outputDirectory=dict(),
    )
    outputs = DicomToNrrdConverter.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
"""Parallel workflow execution via SLURM
"""
import os
import sys

from ...interfaces.base import CommandLine
from .base import GraphPluginBase, logger


def node_completed_status(checknode):
    """
    A function to determine if a node has previously completed it's work
    :param checknode: The node to check the run status
    :return: boolean value True indicates that the node does not need to be run.
    """
    """ TODO: place this in the base.py file and refactor """
    node_state_does_not_require_overwrite = checknode.overwrite is False or (
        checknode.overwrite is None and not checknode._interface.always_run
    )
    hash_exists = False
    try:
        hash_exists, _, _, _ = checknode.hash_exists()
    except Exception:
        hash_exists = False
    return hash_exists and node_state_does_not_require_overwrite


class SLURMGraphPlugin(GraphPluginBase):
    """Execute using SLURM

    The plugin_args input to run can be used to control the SGE execution.
    Currently supported options are:

    - template : template to use for batch job submission
    - qsub_args : arguments to be prepended to the job execution script in the
                  qsub call

    """

    _template = "#!/bin/bash"

    def __init__(self, **kwargs):
        self._sbatch_args = ""
        if "plugin_args" in kwargs and kwargs["plugin_args"]:
            if "retry_timeout" in kwargs["plugin_args"]:
                self._retry_timeout = kwargs["plugin_args"]["retry_timeout"]
            if "max_tries" in kwargs["plugin_args"]:
                self._max_tries = kwargs["plugin_args"]["max_tries"]
            if "template" in kwargs["plugin_args"]:
                self._template = kwargs["plugin_args"]["template"]
                if os.path.isfile(self._template):
                    self._template = open(self._template).read()
            if "sbatch_args" in kwargs["plugin_args"]:
                self._sbatch_args = kwargs["plugin_args"]["sbatch_args"]
            if "dont_resubmit_completed_jobs" in kwargs["plugin_args"]:
                self._dont_resubmit_completed_jobs = kwargs["plugin_args"][
                    "dont_resubmit_completed_jobs"
                ]
            else:
                self._dont_resubmit_completed_jobs = False
        super(SLURMGraphPlugin, self).__init__(**kwargs)

    def _submit_graph(self, pyfiles, dependencies, nodes):
        def make_job_name(jobnumber, nodeslist):
            """
            - jobnumber: The index number of the job to create
            - nodeslist: The name of the node being processed
            - return: A string representing this job to be displayed by SLURM
            """
            job_name = "j{0}_{1}".format(jobnumber, nodeslist[jobnumber]._id)
            # Condition job_name to be a valid bash identifier (i.e. - is invalid)
            job_name = job_name.replace("-", "_").replace(".", "_").replace(":", "_")
            return job_name

        batch_dir, _ = os.path.split(pyfiles[0])
        submitjobsfile = os.path.join(batch_dir, "submit_jobs.sh")

        cache_doneness_per_node = dict()
        if (
            self._dont_resubmit_completed_jobs
        ):  # A future parameter for controlling this behavior could be added here
            for idx, pyscript in enumerate(pyfiles):
                node = nodes[idx]
                node_status_done = node_completed_status(node)

                # if the node itself claims done, then check to ensure all
                # dependancies are also done
                if node_status_done and idx in dependencies:
                    for child_idx in dependencies[idx]:
                        if child_idx in cache_doneness_per_node:
                            child_status_done = cache_doneness_per_node[child_idx]
                        else:
                            child_status_done = node_completed_status(nodes[child_idx])
                        node_status_done = node_status_done and child_status_done

                cache_doneness_per_node[idx] = node_status_done

        with open(submitjobsfile, "wt") as fp:
            fp.writelines("#!/usr/bin/env bash\n")
            fp.writelines("# Condense format attempted\n")
            for idx, pyscript in enumerate(pyfiles):
                node = nodes[idx]
                if cache_doneness_per_node.get(idx, False):
                    continue
                else:
                    template, sbatch_args = self._get_args(
                        node, ["template", "sbatch_args"]
                    )

                    batch_dir, name = os.path.split(pyscript)
                    name = ".".join(name.split(".")[:-1])
                    batchscript = "\n".join(
                        (template, "%s %s" % (sys.executable, pyscript))
                    )
                    batchscriptfile = os.path.join(
                        batch_dir, "batchscript_%s.sh" % name
                    )

                    batchscriptoutfile = batchscriptfile + ".o"
                    batchscripterrfile = batchscriptfile + ".e"

                    with open(batchscriptfile, "wt") as batchfp:
                        batchfp.writelines(batchscript)
                        batchfp.close()
                    deps = ""
                    if idx in dependencies:
                        values = ""
                        for jobid in dependencies[idx]:
                            # Avoid dependancies of done jobs
                            if (
                                not self._dont_resubmit_completed_jobs
                                or not cache_doneness_per_node[jobid]
                            ):
                                values += "${{{0}}}:".format(
                                    make_job_name(jobid, nodes)
                                )
                        if (
                            values != ""
                        ):  # i.e. if some jobs were added to dependency list
                            values = values.rstrip(":")
                            deps = "--dependency=afterok:%s" % values
                    jobname = make_job_name(idx, nodes)
                    # Do not use default output locations if they are set in self._sbatch_args
                    stderrFile = ""
                    if self._sbatch_args.count("-e ") == 0:
                        stderrFile = "-e {errFile}".format(errFile=batchscripterrfile)
                    stdoutFile = ""
                    if self._sbatch_args.count("-o ") == 0:
                        stdoutFile = "-o {outFile}".format(outFile=batchscriptoutfile)
                    full_line = "{jobNm}=$(sbatch {outFileOption} {errFileOption} {extraSBatchArgs} {dependantIndex} -J {jobNm} {batchscript} | awk '/^Submitted/ {{print $4}}')\n".format(
                        jobNm=jobname,
                        outFileOption=stdoutFile,
                        errFileOption=stderrFile,
                        extraSBatchArgs=sbatch_args,
                        dependantIndex=deps,
                        batchscript=batchscriptfile,
                    )
                    fp.writelines(full_line)
        cmd = CommandLine(
            "bash",
            environ=dict(os.environ),
            resource_monitor=False,
            terminal_output="allatonce",
        )
        cmd.inputs.args = "%s" % submitjobsfile
        cmd.run()
        logger.info("submitted all jobs to queue")



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..dti import DTLUTGen


def test_DTLUTGen_inputs():
    input_map = dict(
        acg=dict(
            argstr="-acg",
        ),
        args=dict(
            argstr="%s",
        ),
        bingham=dict(
            argstr="-bingham",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        frange=dict(
            argstr="-frange %s",
            position=1,
            units="NA",
        ),
        inversion=dict(
            argstr="-inversion %d",
            units="NA",
        ),
        lrange=dict(
            argstr="-lrange %s",
            position=1,
            units="NA",
        ),
        out_file=dict(
            argstr="> %s",
            extensions=None,
            genfile=True,
            position=-1,
        ),
        samples=dict(
            argstr="-samples %d",
            units="NA",
        ),
        scheme_file=dict(
            argstr="-schemefile %s",
            extensions=None,
            mandatory=True,
            position=2,
        ),
        snr=dict(
            argstr="-snr %f",
            units="NA",
        ),
        step=dict(
            argstr="-step %f",
            units="NA",
        ),
        trace=dict(
            argstr="-trace %G",
            units="NA",
        ),
        watson=dict(
            argstr="-watson",
        ),
    )
    inputs = DTLUTGen.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_DTLUTGen_outputs():
    output_map = dict(
        dtLUT=dict(
            extensions=None,
        ),
    )
    outputs = DTLUTGen.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)
import os


class ACPCTransformInputSpec(CommandLineInputSpec):
    acpc = InputMultiPath(
        traits.List(traits.Float(), minlen=3, maxlen=3),
        desc="ACPC line, two fiducial points, one at the anterior commissure and one at the posterior commissure.",
        argstr="--acpc %s...",
    )
    midline = InputMultiPath(
        traits.List(traits.Float(), minlen=3, maxlen=3),
        desc="The midline is a series of points defining the division between the hemispheres of the brain (the mid sagittal plane).",
        argstr="--midline %s...",
    )
    outputTransform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="A transform filled in from the ACPC and Midline registration calculation",
        argstr="--outputTransform %s",
    )
    debugSwitch = traits.Bool(
        desc="Click if wish to see debugging output", argstr="--debugSwitch "
    )


class ACPCTransformOutputSpec(TraitedSpec):
    outputTransform = File(
        desc="A transform filled in from the ACPC and Midline registration calculation",
        exists=True,
    )


class ACPCTransform(SEMLikeCommandLine):
    """title: ACPC Transform

category: Registration.Specialized

description: <p>Calculate a transformation from two lists of fiducial points.</p><p>ACPC line is two fiducial points, one at the anterior commissure and one at the posterior commissure. The resulting transform will bring the line connecting them to horizontal to the AP axis.</p><p>The midline is a series of points defining the division between the hemispheres of the brain (the mid sagittal plane). The resulting transform will put the output volume with the mid sagittal plane lined up with the AS plane.</p><p>Use the Filtering module<b>Resample Scalar/Vector/DWI Volume</b>to apply the transformation to a volume.</p>

version: 1.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/ACPCTransform

license: slicer3

contributor: Nicole Aucoin (SPL, BWH), Ron Kikinis (SPL, BWH)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = ACPCTransformInputSpec
    output_spec = ACPCTransformOutputSpec
    _cmd = "ACPCTransform "
    _outputs_filenames = {"outputTransform": "outputTransform.mat"}


class FiducialRegistrationInputSpec(CommandLineInputSpec):
    fixedLandmarks = InputMultiPath(
        traits.List(traits.Float(), minlen=3, maxlen=3),
        desc="Ordered list of landmarks in the fixed image",
        argstr="--fixedLandmarks %s...",
    )
    movingLandmarks = InputMultiPath(
        traits.List(traits.Float(), minlen=3, maxlen=3),
        desc="Ordered list of landmarks in the moving image",
        argstr="--movingLandmarks %s...",
    )
    saveTransform = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Save the transform that results from registration",
        argstr="--saveTransform %s",
    )
    transformType = traits.Enum(
        "Translation",
        "Rigid",
        "Similarity",
        desc="Type of transform to produce",
        argstr="--transformType %s",
    )
    rms = traits.Float(desc="Display RMS Error.", argstr="--rms %f")
    outputMessage = traits.Str(
        desc="Provides more information on the output", argstr="--outputMessage %s"
    )


class FiducialRegistrationOutputSpec(TraitedSpec):
    saveTransform = File(
        desc="Save the transform that results from registration", exists=True
    )


class FiducialRegistration(SEMLikeCommandLine):
    """title: Fiducial Registration

category: Registration.Specialized

description: Computes a rigid, similarity or affine transform from a matched list of fiducials

version: 0.1.0.$Revision$

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/TransformFromFiducials

contributor: Casey B Goodlett (Kitware), Dominik Meier (SPL, BWH)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = FiducialRegistrationInputSpec
    output_spec = FiducialRegistrationOutputSpec
    _cmd = "FiducialRegistration "
    _outputs_filenames = {"saveTransform": "saveTransform.txt"}


class VBRAINSDemonWarpInputSpec(CommandLineInputSpec):
    movingVolume = InputMultiPath(
        File(exists=True),
        desc="Required: input moving image",
        argstr="--movingVolume %s...",
    )
    fixedVolume = InputMultiPath(
        File(exists=True),
        desc="Required: input fixed (target) image",
        argstr="--fixedVolume %s...",
    )
    inputPixelType = traits.Enum(
        "float",
        "short",
        "ushort",
        "int",
        "uchar",
        desc="Input volumes will be typecast to this format: float|short|ushort|int|uchar",
        argstr="--inputPixelType %s",
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output resampled moving image (will have the same physical space as the fixedVolume).",
        argstr="--outputVolume %s",
    )
    outputDisplacementFieldVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output deformation field vector image (will have the same physical space as the fixedVolume).",
        argstr="--outputDisplacementFieldVolume %s",
    )
    outputPixelType = traits.Enum(
        "float",
        "short",
        "ushort",
        "int",
        "uchar",
        desc="outputVolume will be typecast to this format: float|short|ushort|int|uchar",
        argstr="--outputPixelType %s",
    )
    interpolationMode = traits.Enum(
        "NearestNeighbor",
        "Linear",
        "ResampleInPlace",
        "BSpline",
        "WindowedSinc",
        "Hamming",
        "Cosine",
        "Welch",
        "Lanczos",
        "Blackman",
        desc="Type of interpolation to be used when applying transform to moving volume.  Options are Linear, ResampleInPlace, NearestNeighbor, BSpline, or WindowedSinc",
        argstr="--interpolationMode %s",
    )
    registrationFilterType = traits.Enum(
        "Demons",
        "FastSymmetricForces",
        "Diffeomorphic",
        "LogDemons",
        "SymmetricLogDemons",
        desc="Registration Filter Type: Demons|FastSymmetricForces|Diffeomorphic|LogDemons|SymmetricLogDemons",
        argstr="--registrationFilterType %s",
    )
    smoothDisplacementFieldSigma = traits.Float(
        desc="A gaussian smoothing value to be applied to the deformation feild at each iteration.",
        argstr="--smoothDisplacementFieldSigma %f",
    )
    numberOfPyramidLevels = traits.Int(
        desc="Number of image pyramid levels to use in the multi-resolution registration.",
        argstr="--numberOfPyramidLevels %d",
    )
    minimumFixedPyramid = InputMultiPath(
        traits.Int,
        desc="The shrink factor for the first level of the fixed image pyramid. (i.e. start at 1/16 scale, then 1/8, then 1/4, then 1/2, and finally full scale)",
        sep=",",
        argstr="--minimumFixedPyramid %s",
    )
    minimumMovingPyramid = InputMultiPath(
        traits.Int,
        desc="The shrink factor for the first level of the moving image pyramid. (i.e. start at 1/16 scale, then 1/8, then 1/4, then 1/2, and finally full scale)",
        sep=",",
        argstr="--minimumMovingPyramid %s",
    )
    arrayOfPyramidLevelIterations = InputMultiPath(
        traits.Int,
        desc="The number of iterations for each pyramid level",
        sep=",",
        argstr="--arrayOfPyramidLevelIterations %s",
    )
    histogramMatch = traits.Bool(
        desc="Histogram Match the input images.  This is suitable for images of the same modality that may have different absolute scales, but the same overall intensity profile.",
        argstr="--histogramMatch ",
    )
    numberOfHistogramBins = traits.Int(
        desc="The number of histogram levels", argstr="--numberOfHistogramBins %d"
    )
    numberOfMatchPoints = traits.Int(
        desc="The number of match points for histrogramMatch",
        argstr="--numberOfMatchPoints %d",
    )
    medianFilterSize = InputMultiPath(
        traits.Int,
        desc="Median filter radius in all 3 directions.  When images have a lot of salt and pepper noise, this step can improve the registration.",
        sep=",",
        argstr="--medianFilterSize %s",
    )
    initializeWithDisplacementField = File(
        desc="Initial deformation field vector image file name",
        exists=True,
        argstr="--initializeWithDisplacementField %s",
    )
    initializeWithTransform = File(
        desc="Initial Transform filename",
        exists=True,
        argstr="--initializeWithTransform %s",
    )
    makeBOBF = traits.Bool(
        desc="Flag to make Brain-Only Background-Filled versions of the input and target volumes.",
        argstr="--makeBOBF ",
    )
    fixedBinaryVolume = File(
        desc="Mask filename for desired region of interest in the Fixed image.",
        exists=True,
        argstr="--fixedBinaryVolume %s",
    )
    movingBinaryVolume = File(
        desc="Mask filename for desired region of interest in the Moving image.",
        exists=True,
        argstr="--movingBinaryVolume %s",
    )
    lowerThresholdForBOBF = traits.Int(
        desc="Lower threshold for performing BOBF", argstr="--lowerThresholdForBOBF %d"
    )
    upperThresholdForBOBF = traits.Int(
        desc="Upper threshold for performing BOBF", argstr="--upperThresholdForBOBF %d"
    )
    backgroundFillValue = traits.Int(
        desc="Replacement value to overwrite background when performing BOBF",
        argstr="--backgroundFillValue %d",
    )
    seedForBOBF = InputMultiPath(
        traits.Int,
        desc="coordinates in all 3 directions for Seed when performing BOBF",
        sep=",",
        argstr="--seedForBOBF %s",
    )
    neighborhoodForBOBF = InputMultiPath(
        traits.Int,
        desc="neighborhood in all 3 directions to be included when performing BOBF",
        sep=",",
        argstr="--neighborhoodForBOBF %s",
    )
    outputDisplacementFieldPrefix = traits.Str(
        desc="Displacement field filename prefix for writing separate x, y, and z component images",
        argstr="--outputDisplacementFieldPrefix %s",
    )
    outputCheckerboardVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Genete a checkerboard image volume between the fixedVolume and the deformed movingVolume.",
        argstr="--outputCheckerboardVolume %s",
    )
    checkerboardPatternSubdivisions = InputMultiPath(
        traits.Int,
        desc="Number of Checkerboard subdivisions in all 3 directions",
        sep=",",
        argstr="--checkerboardPatternSubdivisions %s",
    )
    outputNormalized = traits.Bool(
        desc="Flag to warp and write the normalized images to output.  In normalized images the image values are fit-scaled to be between 0 and the maximum storage type value.",
        argstr="--outputNormalized ",
    )
    outputDebug = traits.Bool(
        desc="Flag to write debugging images after each step.", argstr="--outputDebug "
    )
    weightFactors = InputMultiPath(
        traits.Float,
        desc="Weight fatctors for each input images",
        sep=",",
        argstr="--weightFactors %s",
    )
    gradient_type = traits.Enum(
        "0",
        "1",
        "2",
        desc="Type of gradient used for computing the demons force (0 is symmetrized, 1 is fixed image, 2 is moving image)",
        argstr="--gradient_type %s",
    )
    upFieldSmoothing = traits.Float(
        desc="Smoothing sigma for the update field at each iteration",
        argstr="--upFieldSmoothing %f",
    )
    max_step_length = traits.Float(
        desc="Maximum length of an update vector (0: no restriction)",
        argstr="--max_step_length %f",
    )
    use_vanilla_dem = traits.Bool(
        desc="Run vanilla demons algorithm", argstr="--use_vanilla_dem "
    )
    gui = traits.Bool(
        desc="Display intermediate image volumes for debugging", argstr="--gui "
    )
    promptUser = traits.Bool(
        desc="Prompt the user to hit enter each time an image is sent to the DebugImageViewer",
        argstr="--promptUser ",
    )
    numberOfBCHApproximationTerms = traits.Int(
        desc="Number of terms in the BCH expansion",
        argstr="--numberOfBCHApproximationTerms %d",
    )
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d",
    )


class VBRAINSDemonWarpOutputSpec(TraitedSpec):
    outputVolume = File(
        desc="Required: output resampled moving image (will have the same physical space as the fixedVolume).",
        exists=True,
    )
    outputDisplacementFieldVolume = File(
        desc="Output deformation field vector image (will have the same physical space as the fixedVolume).",
        exists=True,
    )
    outputCheckerboardVolume = File(
        desc="Genete a checkerboard image volume between the fixedVolume and the deformed movingVolume.",
        exists=True,
    )


class VBRAINSDemonWarp(SEMLikeCommandLine):
    """title: Vector Demon Registration (BRAINS)

category: Registration.Specialized

description:
    This program finds a deformation field to warp a moving image onto a fixed image.  The images must be of the same signal kind, and contain an image of the same kind of object.  This program uses the Thirion Demons warp software in ITK, the Insight Toolkit.  Additional information is available at: http://www.nitrc.org/projects/brainsdemonwarp.



version: 3.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:BRAINSDemonWarp

license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

contributor: This tool was developed by Hans J. Johnson and Greg Harris.

acknowledgements: The development of this tool was supported by funding from grants NS050568 and NS40068 from the National Institute of Neurological Disorders and Stroke and grants MH31593, MH40856, from the National Institute of Mental Health.

"""

    input_spec = VBRAINSDemonWarpInputSpec
    output_spec = VBRAINSDemonWarpOutputSpec
    _cmd = "VBRAINSDemonWarp "
    _outputs_filenames = {
        "outputVolume": "outputVolume.nii",
        "outputCheckerboardVolume": "outputCheckerboardVolume.nii",
        "outputDisplacementFieldVolume": "outputDisplacementFieldVolume.nrrd",
    }


class BRAINSDemonWarpInputSpec(CommandLineInputSpec):
    movingVolume = File(
        desc="Required: input moving image", exists=True, argstr="--movingVolume %s"
    )
    fixedVolume = File(
        desc="Required: input fixed (target) image",
        exists=True,
        argstr="--fixedVolume %s",
    )
    inputPixelType = traits.Enum(
        "float",
        "short",
        "ushort",
        "int",
        "uchar",
        desc="Input volumes will be typecast to this format: float|short|ushort|int|uchar",
        argstr="--inputPixelType %s",
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output resampled moving image (will have the same physical space as the fixedVolume).",
        argstr="--outputVolume %s",
    )
    outputDisplacementFieldVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Output deformation field vector image (will have the same physical space as the fixedVolume).",
        argstr="--outputDisplacementFieldVolume %s",
    )
    outputPixelType = traits.Enum(
        "float",
        "short",
        "ushort",
        "int",
        "uchar",
        desc="outputVolume will be typecast to this format: float|short|ushort|int|uchar",
        argstr="--outputPixelType %s",
    )
    interpolationMode = traits.Enum(
        "NearestNeighbor",
        "Linear",
        "ResampleInPlace",
        "BSpline",
        "WindowedSinc",
        "Hamming",
        "Cosine",
        "Welch",
        "Lanczos",
        "Blackman",
        desc="Type of interpolation to be used when applying transform to moving volume.  Options are Linear, ResampleInPlace, NearestNeighbor, BSpline, or WindowedSinc",
        argstr="--interpolationMode %s",
    )
    registrationFilterType = traits.Enum(
        "Demons",
        "FastSymmetricForces",
        "Diffeomorphic",
        desc="Registration Filter Type: Demons|FastSymmetricForces|Diffeomorphic",
        argstr="--registrationFilterType %s",
    )
    smoothDisplacementFieldSigma = traits.Float(
        desc="A gaussian smoothing value to be applied to the deformation feild at each iteration.",
        argstr="--smoothDisplacementFieldSigma %f",
    )
    numberOfPyramidLevels = traits.Int(
        desc="Number of image pyramid levels to use in the multi-resolution registration.",
        argstr="--numberOfPyramidLevels %d",
    )
    minimumFixedPyramid = InputMultiPath(
        traits.Int,
        desc="The shrink factor for the first level of the fixed image pyramid. (i.e. start at 1/16 scale, then 1/8, then 1/4, then 1/2, and finally full scale)",
        sep=",",
        argstr="--minimumFixedPyramid %s",
    )
    minimumMovingPyramid = InputMultiPath(
        traits.Int,
        desc="The shrink factor for the first level of the moving image pyramid. (i.e. start at 1/16 scale, then 1/8, then 1/4, then 1/2, and finally full scale)",
        sep=",",
        argstr="--minimumMovingPyramid %s",
    )
    arrayOfPyramidLevelIterations = InputMultiPath(
        traits.Int,
        desc="The number of iterations for each pyramid level",
        sep=",",
        argstr="--arrayOfPyramidLevelIterations %s",
    )
    histogramMatch = traits.Bool(
        desc="Histogram Match the input images.  This is suitable for images of the same modality that may have different absolute scales, but the same overall intensity profile.",
        argstr="--histogramMatch ",
    )
    numberOfHistogramBins = traits.Int(
        desc="The number of histogram levels", argstr="--numberOfHistogramBins %d"
    )
    numberOfMatchPoints = traits.Int(
        desc="The number of match points for histrogramMatch",
        argstr="--numberOfMatchPoints %d",
    )
    medianFilterSize = InputMultiPath(
        traits.Int,
        desc="Median filter radius in all 3 directions.  When images have a lot of salt and pepper noise, this step can improve the registration.",
        sep=",",
        argstr="--medianFilterSize %s",
    )
    initializeWithDisplacementField = File(
        desc="Initial deformation field vector image file name",
        exists=True,
        argstr="--initializeWithDisplacementField %s",
    )
    initializeWithTransform = File(
        desc="Initial Transform filename",
        exists=True,
        argstr="--initializeWithTransform %s",
    )
    maskProcessingMode = traits.Enum(
        "NOMASK",
        "ROIAUTO",
        "ROI",
        "BOBF",
        desc="What mode to use for using the masks: NOMASK|ROIAUTO|ROI|BOBF.  If ROIAUTO is choosen, then the mask is implicitly defined using a otsu forground and hole filling algorithm. Where the Region Of Interest mode uses the masks to define what parts of the image should be used for computing the deformation field.  Brain Only Background Fill uses the masks to pre-process the input images by clipping and filling in the background with a predefined value.",
        argstr="--maskProcessingMode %s",
    )
    fixedBinaryVolume = File(
        desc="Mask filename for desired region of interest in the Fixed image.",
        exists=True,
        argstr="--fixedBinaryVolume %s",
    )
    movingBinaryVolume = File(
        desc="Mask filename for desired region of interest in the Moving image.",
        exists=True,
        argstr="--movingBinaryVolume %s",
    )
    lowerThresholdForBOBF = traits.Int(
        desc="Lower threshold for performing BOBF", argstr="--lowerThresholdForBOBF %d"
    )
    upperThresholdForBOBF = traits.Int(
        desc="Upper threshold for performing BOBF", argstr="--upperThresholdForBOBF %d"
    )
    backgroundFillValue = traits.Int(
        desc="Replacement value to overwrite background when performing BOBF",
        argstr="--backgroundFillValue %d",
    )
    seedForBOBF = InputMultiPath(
        traits.Int,
        desc="coordinates in all 3 directions for Seed when performing BOBF",
        sep=",",
        argstr="--seedForBOBF %s",
    )
    neighborhoodForBOBF = InputMultiPath(
        traits.Int,
        desc="neighborhood in all 3 directions to be included when performing BOBF",
        sep=",",
        argstr="--neighborhoodForBOBF %s",
    )
    outputDisplacementFieldPrefix = traits.Str(
        desc="Displacement field filename prefix for writing separate x, y, and z component images",
        argstr="--outputDisplacementFieldPrefix %s",
    )
    outputCheckerboardVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Genete a checkerboard image volume between the fixedVolume and the deformed movingVolume.",
        argstr="--outputCheckerboardVolume %s",
    )
    checkerboardPatternSubdivisions = InputMultiPath(
        traits.Int,
        desc="Number of Checkerboard subdivisions in all 3 directions",
        sep=",",
        argstr="--checkerboardPatternSubdivisions %s",
    )
    outputNormalized = traits.Bool(
        desc="Flag to warp and write the normalized images to output.  In normalized images the image values are fit-scaled to be between 0 and the maximum storage type value.",
        argstr="--outputNormalized ",
    )
    outputDebug = traits.Bool(
        desc="Flag to write debugging images after each step.", argstr="--outputDebug "
    )
    gradient_type = traits.Enum(
        "0",
        "1",
        "2",
        desc="Type of gradient used for computing the demons force (0 is symmetrized, 1 is fixed image, 2 is moving image)",
        argstr="--gradient_type %s",
    )
    upFieldSmoothing = traits.Float(
        desc="Smoothing sigma for the update field at each iteration",
        argstr="--upFieldSmoothing %f",
    )
    max_step_length = traits.Float(
        desc="Maximum length of an update vector (0: no restriction)",
        argstr="--max_step_length %f",
    )
    use_vanilla_dem = traits.Bool(
        desc="Run vanilla demons algorithm", argstr="--use_vanilla_dem "
    )
    gui = traits.Bool(
        desc="Display intermediate image volumes for debugging", argstr="--gui "
    )
    promptUser = traits.Bool(
        desc="Prompt the user to hit enter each time an image is sent to the DebugImageViewer",
        argstr="--promptUser ",
    )
    numberOfBCHApproximationTerms = traits.Int(
        desc="Number of terms in the BCH expansion",
        argstr="--numberOfBCHApproximationTerms %d",
    )
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d",
    )


class BRAINSDemonWarpOutputSpec(TraitedSpec):
    outputVolume = File(
        desc="Required: output resampled moving image (will have the same physical space as the fixedVolume).",
        exists=True,
    )
    outputDisplacementFieldVolume = File(
        desc="Output deformation field vector image (will have the same physical space as the fixedVolume).",
        exists=True,
    )
    outputCheckerboardVolume = File(
        desc="Genete a checkerboard image volume between the fixedVolume and the deformed movingVolume.",
        exists=True,
    )


class BRAINSDemonWarp(SEMLikeCommandLine):
    """title: Demon Registration (BRAINS)

category: Registration.Specialized

description:
    This program finds a deformation field to warp a moving image onto a fixed image.  The images must be of the same signal kind, and contain an image of the same kind of object.  This program uses the Thirion Demons warp software in ITK, the Insight Toolkit.  Additional information is available at: http://www.nitrc.org/projects/brainsdemonwarp.



version: 3.0.0

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Modules:BRAINSDemonWarp

license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

contributor: This tool was developed by Hans J. Johnson and Greg Harris.

acknowledgements: The development of this tool was supported by funding from grants NS050568 and NS40068 from the National Institute of Neurological Disorders and Stroke and grants MH31593, MH40856, from the National Institute of Mental Health.

"""

    input_spec = BRAINSDemonWarpInputSpec
    output_spec = BRAINSDemonWarpOutputSpec
    _cmd = "BRAINSDemonWarp "
    _outputs_filenames = {
        "outputVolume": "outputVolume.nii",
        "outputCheckerboardVolume": "outputCheckerboardVolume.nii",
        "outputDisplacementFieldVolume": "outputDisplacementFieldVolume.nrrd",
    }



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..analysis import CoherenceAnalyzer


def test_CoherenceAnalyzer_inputs():
    input_map = dict(
        NFFT=dict(
            usedefault=True,
        ),
        TR=dict(),
        figure_type=dict(
            usedefault=True,
        ),
        frequency_range=dict(
            usedefault=True,
        ),
        in_TS=dict(),
        in_file=dict(
            extensions=None,
            requires=("TR",),
        ),
        n_overlap=dict(
            usedefault=True,
        ),
        output_csv_file=dict(
            extensions=None,
        ),
        output_figure_file=dict(
            extensions=None,
        ),
    )
    inputs = CoherenceAnalyzer.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_CoherenceAnalyzer_outputs():
    output_map = dict(
        coherence_array=dict(),
        coherence_csv=dict(
            extensions=None,
        ),
        coherence_fig=dict(
            extensions=None,
        ),
        timedelay_array=dict(),
        timedelay_csv=dict(
            extensions=None,
        ),
        timedelay_fig=dict(
            extensions=None,
        ),
    )
    outputs = CoherenceAnalyzer.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
